@@ -23,11 +23,7 @@ except:
 
 # KForge Trac handler
 try:
-    from trac.web.modpython_frontend import handler as trachandler
+    from kforge.handlers.tracmodpython import handler as trachandler
 except:
-    try:
-       # For Trac version < v0.9.
-       import trac.ModPythonHandler as trachandler
-    except:
-       pass
+    pass
 
