@@ -21,13 +21,14 @@ class TestProjectAccessController(TestCase):
     def tearDown(self):
         self.ac = None
 
-    def isAuthorised(self):
-        return self.ac.isAuthorised(
-            person=self.person, 
-            actionName=self.actionName, 
-            protectedObject=self.object, 
-            project=self.registry.projects['administration'],
-        )
+    # Commented out, since this method is not being used in any test.
+    #def isAccessAuthorised(self):
+    #    return self.ac.isAccessAuthorised(
+    #        person=self.person, 
+    #        actionName=self.actionName, 
+    #        protectedObject=self.object, 
+    #        project=self.registry.projects['administration'],
+    #    )
     
     def test_setUp(self):
         self.failUnless(self.ac)
