@@ -13,7 +13,8 @@ features.allowReplace = True
 class TestApplication(unittest.TestCase):
 
     def setUp(self):
-        self.application = Application()
+        import kforge.soleInstance
+        self.application = kforge.soleInstance.application
 
     def tearDown(self):
         self.application = None
