@@ -35,13 +35,9 @@ class ApplicationBuilder(dm.builder.ApplicationBuilder):
 
     def findFileSystem(self):
         import kforge.filesystem
-        return kforge.filesystem.FileSystemPathBuilder()
+        return kforge.filesystem.FileSystem()
 
     def findAccessController(self):
         import kforge.accesscontrol
         return kforge.accesscontrol.ProjectAccessController()
 
-    def findPluginFactory(self):
-        import kforge.plugin.factory
-        return kforge.plugin.factory.PluginFactory()
-
