@@ -6,6 +6,10 @@ from subprocess import Popen, PIPE
 import re
 import kforge.regexps
 from threading import Thread
+from threading import Condition
+import time
+from kforge.dictionarywords import *
+import traceback
 
 # Todo: Test for cloning after adding changesets (Mercurial, Git, and Subversion).
 # Todo: Test for pulling adding revisions from another clone (Mercurial, Git, and Subversion).
@@ -34,18 +38,18 @@ class SshCommandRunner(object):
                 assert sshKeyId
             except:
                 msg = "SSH handler has bad configuration (key ID not given)."
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (1)"
 
             sshKeys = application.registry.sshKeys.findDomainObjects(id=sshKeyId)
             if not sshKeys:
                 msg = "SSH handler has bad configuration (key ID '%s' not found in register)." % sshKeyId
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (2)"
 
             if len(sshKeys) > 1:
                 msg = "SSH handler discovered model error (more than one ssh key for ID '%s')." % sshKeyId
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (3)"
 
             sshKey = sshKeys[0]
@@ -56,17 +60,17 @@ class SshCommandRunner(object):
                     sshKey.save()
             except Exception, inst:
                 msg = "SSH handler could not update 'isConsummated' attribute of ssh key '%s': %s" % (sshKeyId, inst)
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (4)"
 
             if not sshKey.person:
                 msg = "SSH handler discovered model error (missing person for ssh key '%s')." % sshKeyId
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (5)"
 
             if not sshKey.person.name:
                 msg = "SSH handler discovered model error (missing person name for person ID '%s')." % sshKey.person.id
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Internal server error (6)"
 
             person = sshKey.person
@@ -76,7 +80,7 @@ class SshCommandRunner(object):
 
             if not sshRequest:
                 msg = "SSH command not found in request from person '%s'." % personName
-                application.logger.warning(msg)
+                application.logger.error(msg)
                 raise Exception, "Bad request"
                 
             application.logger.info("SSH request from person '%s' (key '%s'): %s" % (personName, sshKeyId, sshRequest))
@@ -106,7 +110,8 @@ class SshCommandRunner(object):
                 application.logger.info("Access denied for SSH request from '%s': %s: %s." % (personName, sshRequest, inst))
                 raise Exception, "kforge: Access denied"
             except Exception, inst:
-                application.logger.warning("Error processing SSH request from '%s': %s: %s." % (personName, sshRequest, inst))
+                trace = traceback.format_exc()
+                application.logger.error("Error processing SSH request from '%s': %s: %s: %s" % (personName, sshRequest, inst, trace))
                 raise Exception, "kforge: Internal server error (7)"
 
         except Exception, inst:
@@ -122,7 +127,9 @@ class BaseCommand(object):
     def __init__(self, sshRequest, application=None, person=None):
         self.sshRequest = sshRequest
         self.application = application
+        self.logger = self.application.logger
         self.person = person
+        self.isDebug = 'DEBUG' in self.application.dictionary[LOG_LEVEL]
 
     def run(self):
         self.splitSshRequest()
@@ -167,7 +174,7 @@ class BaseCommand(object):
         except Exception, inst:
             raise ServiceNotFound, inst
         self.service = service
-        self.application.logger.info("SSH request for access to service: %s" % self.service)
+        self.log("SSH request for access to service: %s" % self.service)
         self.servicePath = self.application.fileSystem.getServicePath(self.service)
 
     def rewriteRequestArgs(self):
@@ -192,16 +199,15 @@ class BaseCommand(object):
         self.shellCmd = self.sshRequestCmd + " " + " ".join(self.sshRequestArgs)
 
     def call(self):
-        try:
-            self.application.logger.info("Executing command via SSH: %s" % self.shellCmd)
-            returnCode = call(self.shellCmd, shell=True)
-            if returnCode < 0:
-                raise Exception, "SSH command was terminated by signal: %s" % (returnCode)
-            elif returnCode > 0:
-                raise Exception, "SSH command exited with non-zero error code: %s" % (returnCode)
-            self.application.logger.debug("SSH command executed OK")
-        except OSError, inst:
-            raise Exception, "SSH command execution failed: %s" % inst
+        raise Exception, "Method not implemented on %s" % self.__class__
+
+    def log(self, line, isDebug=False, isError=False):
+        if isDebug:
+            self.logger.debug(line)
+        elif isError:
+            self.logger.error(line)
+        else:
+            self.logger.info(line)
 
 
 class GitCommand(BaseCommand):
@@ -245,24 +251,37 @@ class GitCommand(BaseCommand):
         else:
             self.sshRequestArgs[0] = self.servicePath
 
+    def call(self):
+        try:
+            self.application.logger.info("Executing Git SSH command: %s" % self.shellCmd)
+            self.application.logger.debug("Git client is authorised, so allowing unmediated communications on STDIN and STDOUT.")
+            returnCode = call(self.shellCmd, shell=True)
+            if returnCode < 0:
+                raise Exception, "Git SSH command was terminated by signal: %s" % (returnCode)
+            elif returnCode > 0:
+                raise Exception, "Git SSH command exited with non-zero error code: %s" % (returnCode)
+            self.application.logger.debug("Git SSH command executed OK")
+        except OSError, inst:
+            raise Exception, "Git SSH command execution failed: %s" % inst
+
 
 class MercurialCommand(BaseCommand):
 
     def validateRequest(self):
         if self.sshRequestCmd != 'hg':
-            msg = "Mercurial command '%s' not supported." % self.sshRequestCmd
+            msg = "Mercurial SSH command '%s' not supported." % self.sshRequestCmd
             raise Exception, msg
         if len(self.sshRequestArgs) != 4:
-            msg = "Mercurial command args look wrong (len): %s" % self.sshRequestArgs
+            msg = "Mercurial SSH command args look wrong (len): %s" % self.sshRequestArgs
             raise Exception, msg
         if self.sshRequestArgs[0] != '-R':
-            msg = "Mercurial command args look wrong (0): %s" % self.sshRequestArgs
+            msg = "Mercurial SSH command args look wrong (0): %s" % self.sshRequestArgs
             raise Exception, msg
         if self.sshRequestArgs[2] != 'serve':
-            msg = "Mercurial command args look wrong (2): %s" % self.sshRequestArgs
+            msg = "Mercurial SSH command args look wrong (2): %s" % self.sshRequestArgs
             raise Exception, msg
         if self.sshRequestArgs[3] != '--stdio':
-            msg = "Mercurial command args look wrong (3): %s" % self.sshRequestArgs
+            msg = "Mercurial SSH command args look wrong (3): %s" % self.sshRequestArgs
             raise Exception, msg
 
     def initActionName(self):
@@ -275,15 +294,16 @@ class MercurialCommand(BaseCommand):
         self.sshRequestArgs[1] = os.path.join(self.servicePath, 'repo')
 
     def call(self):
-        self.application.logger.info("Executing command via SSH: %s" % self.shellCmd)
         canUpdate = self.isAccessAuthorised(actionName='Update')
         try:
-            self.server = Popen(self.shellCmd, stdin=PIPE, stdout=PIPE, shell=True)
-            # Start direct client-server communication.
+            self.application.logger.info("Executing Mercurial SSH command: %s" % self.shellCmd)
             if canUpdate:
-                inboundReader = ReadSingleChar
-                validInboundPatterns = []
+                self.application.logger.debug("Mercurial client has update permission, so allowing unmediated communications on STDIN and STDOUT.")
+                stdin = sys.stdin
+                inboundReader = None
             else:
+                self.application.logger.debug("Client has no update permission, so filtering STDIN and allowing unmediated communications on STDOUT only.")
+                stdin = PIPE
                 inboundReader = ReadHgMessage
                 validInboundPatterns = [
                     re.compile('^hello\n$'),
@@ -291,37 +311,40 @@ class MercurialCommand(BaseCommand):
                     re.compile('^heads'),
                     re.compile('^changegroup'),
                     re.compile('^branches'),
+                    # For cloning with hg v1.9.1.
+                    re.compile('^batch'),
+                    re.compile('^cmds'),
+                    re.compile('^listkeys'),
+                    re.compile('^namespace'),
                 ]
-            outboundReader = ReadSingleChar
-            validOutboundPatterns = []
-            stdinThread = PipeThread(name='Pipe inbound', stdin=sys.stdin, stdout=self.server.stdin, reader=inboundReader, logger=self.application.logger, validPatterns=validInboundPatterns)
-            stdoutThread = PipeThread(name='Pipe outbound', stdin=self.server.stdout, stdout=sys.stdout, reader=outboundReader, logger=self.application.logger, validPatterns=validOutboundPatterns)
-            stdinThread.start()
-            stdoutThread.start()
-            #self.application.logger.info("Waiting for server to finish...")
-            returnCode = self.server.wait()
-            #self.application.logger.info("Server  finished...")
-            #self.application.logger.info("Joining on threads....")
-            stdoutThread.join()
-            #self.application.logger.info("Joined on Stdout thread.")
-            stdinThread.join()
-            #self.application.logger.info("Joined on Stdin thread.")
-            if returnCode < 0:
-                raise Exception, "SSH command was terminated by signal: %s" % (returnCode)
-            elif returnCode > 0:
-                raise Exception, "SSH command exited with non-zero error code: %s" % (returnCode)
-            elif stdinThread.error:
-                if isinstance(stdinThread.error, AccessDenied):
-                    raise AccessDenied, "Access denied by inbound thread: %s" % (stdinThread.error)
-
-                else:
-                    raise Exception, "SSH inbound thread exited with error: %s" % (stdinThread.error)
-            elif stdoutThread.error:
-                raise Exception, "SSH outbound thread exited with error: %s" % (stdoutThread.error)
+            self.cmdproc = Popen(self.shellCmd, stdin=stdin, stdout=sys.stdout, shell=True)
+            if inboundReader:
+                inboundThread = PipeThread(name='inbound', stdin=sys.stdin, stdout=self.cmdproc.stdin, readerClass=inboundReader, logger=self.application.logger, validPatterns=validInboundPatterns, isDebug=self.isDebug)
+                inboundThread.start()
             else:
-                self.application.logger.info("SSH command executed OK")
+                inboundThread = None
+            self.application.logger.debug("Waiting for Mercurial SSH command to finish...")
+            returnCode = self.cmdproc.wait()
+            self.application.logger.debug("Mercurial SSH command finished.")
+            if inboundThread:
+                self.application.logger.debug("Joining on inbound thread....")
+                inboundThread.join()
+                self.application.logger.debug("Joined on inbound thread.")
+            if returnCode == None:
+                raise Exception, "Mercurial SSH command was still running. Pid: %s" % (self.cmdproc.pid)
+            if returnCode < 0:
+                raise Exception, "Mercurial SSH command was terminated by signal: %s" % (returnCode)
+            elif returnCode > 0:
+                raise Exception, "Mercurial SSH command exited with non-zero error code: %s" % (returnCode)
+            elif inboundThread and inboundThread.error:
+                if isinstance(inboundThread.error, AccessDenied):
+                    raise AccessDenied, "Access denied by inbound thread: %s" % (inboundThread.error)
+                else:
+                    raise Exception, "Mercurial SSH inbound thread exited with error: %s" % (inboundThread.error)
+            else:
+                self.application.logger.info("Mercurial SSH command executed OK.")
         except OSError, inst:
-            raise Exception, "SSH command execution failed: %s" % inst
+            raise Exception, "Mercurial SSH command execution failed: %s" % inst
 
 
 class SubversionCommand(BaseCommand):
@@ -345,29 +368,29 @@ class SubversionCommand(BaseCommand):
         sys.stdin = os.fdopen(sys.stdin.fileno(), 'r', 0)
         sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
         # Send server greeting to the client.
-        self.server = Popen('svnserve -t', stdin=PIPE, stdout=PIPE, shell=True)
+        self.cmdproc = Popen('svnserve -t', stdin=PIPE, stdout=PIPE, shell=True)
         try:
             #self.application.logger.info("Reading svnserve greeting...")
             greeting = self.readServerMessage()
             #greeting = '( success ( 2 2 ( ) ( edit-pipeline svndiff1 absent-entries commit-revprops depth log-revprops partial-replay ) ) ) '
             self.sendClientMessage(greeting)
-            self.application.logger.info("Sent svnserve greeting to client: %s" % greeting)
+            self.application.logger.debug("Sent svnserve greeting to client: %s" % greeting)
         finally:
-            self.server.stdin.close()
-            self.server.wait()
+            self.cmdproc.stdin.close()
+            self.cmdproc.wait()
         self.svnGreetingResponse = self.readClientMessage()
-        self.application.logger.info("Received greeting response from svn client: %s" % self.svnGreetingResponse)
+        self.application.logger.debug("Received greeting response from svn client: %s" % self.svnGreetingResponse)
         result = re.search('svn\+ssh://(\S*)\s', self.svnGreetingResponse)
         if not result:
             raise Exception, "Failed to tease service URL from Subversion client."
         self.serviceUrl = result.group().strip()
-        #self.application.logger.info("Teased out Subversion service URL: '%s'" % self.serviceUrl)
+        #self.application.logger.debug("Teased out Subversion service URL: '%s'" % self.serviceUrl)
         serviceUrlParts = self.serviceUrl.split('/')
         self.urlPath = '/' + '/'.join(serviceUrlParts[3:5])
         self.urlNoPath = '/'.join(serviceUrlParts[:3])
         if not self.urlPath:
             raise Exception, "Failed to tease service URL path from Subversion client."
-        self.application.logger.info("Teased out Subversion service path: %s" % self.urlPath)
+        self.application.logger.debug("Teased out Subversion service path: %s" % self.urlPath)
 
     def rewriteRequestArgs(self):
         self.sshRequestArgs.append('--tunnel-user')
@@ -378,17 +401,17 @@ class SubversionCommand(BaseCommand):
     def call(self):
         self.application.logger.info("Executing command via SSH: %s" % self.shellCmd)
         urlPatternString = '^\d+:' + self.urlNoPath.replace('+', '\\+').replace('.', '\\.')
-        #self.application.logger.warning("URL pattern: %s" % urlPatternString)
+        #self.application.logger.debug("URL pattern: %s" % urlPatternString)
         urlPattern = re.compile(urlPatternString)
         clientSideUrl = self.urlNoPath + self.urlPath
         serverSideUrl = self.urlNoPath + '/%s' % self.service.id
-        #self.application.logger.warning("Client side: %s" % clientSideUrl)
-        #self.application.logger.warning("Server side: %s" % serverSideUrl)
+        #self.application.logger.debug("Client side: %s" % clientSideUrl)
+        #self.application.logger.debug("Server side: %s" % serverSideUrl)
         inboundRewriter = UrlRewriter(urlPattern=urlPattern, oldUrl=clientSideUrl, newUrl=serverSideUrl, logger=self.application.logger)
         outboundRewriter = UrlRewriter(urlPattern=urlPattern, oldUrl=serverSideUrl, newUrl=clientSideUrl, logger=self.application.logger)
         canUpdate = self.isAccessAuthorised(actionName='Update')
         try:
-            self.server = Popen(self.shellCmd, stdin=PIPE, stdout=PIPE, shell=True)
+            self.cmdproc = Popen(self.shellCmd, stdin=PIPE, stdout=PIPE, shell=True)
             # Inject messages to get things going again.
             # Discard greeting from the server (client has already seen it).
             self.readServerMessage()
@@ -410,76 +433,152 @@ class SubversionCommand(BaseCommand):
                     re.compile('^\(\ success .*'),
                 ]
             validOutboundPatterns = []
-            stdinThread = PipeThread(name='Pipe inbound', stdin=sys.stdin, stdout=self.server.stdin, reader=ReadSvnMessage, logger=self.application.logger, rewriter=inboundRewriter, validPatterns=validInboundPatterns)
-            stdoutThread = PipeThread(name='Pipe outbound', stdin=self.server.stdout, stdout=sys.stdout, reader=ReadSvnMessage, logger=self.application.logger, rewriter=outboundRewriter, validPatterns=validOutboundPatterns)
-            stdinThread.start()
-            stdoutThread.start()
+            inboundThread = PipeThread(name='Pipe inbound', stdin=sys.stdin, stdout=self.cmdproc.stdin, readerClass=ReadSvnMessage, logger=self.application.logger, rewriter=inboundRewriter, validPatterns=validInboundPatterns, isDebug=self.isDebug)
+            outboundThread = PipeThread(name='Pipe outbound', stdin=self.cmdproc.stdout, stdout=sys.stdout, readerClass=ReadSvnMessage, logger=self.application.logger, rewriter=outboundRewriter, validPatterns=validOutboundPatterns, isDebug=self.isDebug, abortStdoutMessage='\n')
+            inboundThread.start()
+            outboundThread.start()
             #self.application.logger.info("Waiting for server to finish...")
-            returnCode = self.server.wait()
+            returnCode = self.cmdproc.wait()
             #self.application.logger.info("Server  finished...")
             #self.application.logger.info("Joining on threads....")
-            stdoutThread.join()
+            outboundThread.join()
             #self.application.logger.info("Joined on Stdout thread.")
-            stdinThread.join()
+            inboundThread.join()
             #self.application.logger.info("Joined on Stdin thread.")
             if returnCode < 0:
-                raise Exception, "SSH command was terminated by signal: %s" % (returnCode)
+                raise Exception, "Subversion SSH command was terminated by signal: %s" % (returnCode)
             elif returnCode > 0:
-                raise Exception, "SSH command exited with non-zero error code: %s" % (returnCode)
-            elif stdinThread.error:
-                if isinstance(stdinThread.error, AccessDenied):
-                    raise AccessDenied, "Access denied by inbound thread: %s" % (stdinThread.error)
+                raise Exception, "Subversion SSH command exited with non-zero error code: %s" % (returnCode)
+            elif inboundThread.error:
+                if isinstance(inboundThread.error, AccessDenied):
+                    raise AccessDenied, "Access denied by inbound thread: %s" % (inboundThread.error)
 
                 else:
-                    raise Exception, "SSH inbound thread exited with error: %s" % (stdinThread.error)
-            elif stdoutThread.error:
-                raise Exception, "SSH outbound thread exited with error: %s" % (stdoutThread.error)
+                    raise Exception, "Subversion SSH inbound thread exited with error: %s" % (inboundThread.error)
+            elif outboundThread.error:
+                raise Exception, "Subversion SSH outbound thread exited with error: %s" % (outboundThread.error)
             else:
-                self.application.logger.info("SSH command executed OK")
+                self.application.logger.debug("Subversion SSH command executed OK")
         except OSError, inst:
-            raise Exception, "SSH command execution failed: %s" % inst
+            raise Exception, "Subversion SSH command execution failed: %s" % inst
 
     def readClientMessage(self):
-        message = ReadSvnMessage(sys.stdin, logger=self.application.logger).read()
-        #self.application.logger.info("Received message from client: '%s'" % message)
+        message = ReadSvnMessage(sys.stdin, self).read()
+        #self.application.logger.debug("Received message from client: '%s'" % message)
         return message
 
     def sendClientMessage(self, message):
-        #self.application.logger.info("Sending message to client: '%s'" % message)
+        #self.application.logger.debug("Sending message to client: '%s'" % message)
         sys.stdout.write(message)
 
     def readServerMessage(self):
-        message = ReadSvnMessage(self.server.stdout, logger=self.application.logger).read()
-        #self.application.logger.info("Received message from server: '%s'" % message)
+        message = ReadSvnMessage(self.cmdproc.stdout, self).read()
+        #self.application.logger.debug("Received message from server: '%s'" % message)
         return message
 
     def sendServerMessage(self, message):
-        #self.application.logger.info("Sending message to server: '%s'" % message)
-        self.server.stdin.write(message)
+        #self.application.logger.debug("Sending message to server: '%s'" % message)
+        self.cmdproc.stdin.write(message)
 
 
 
+class WriteBuffer(Thread):
+
+    def __init__(self, pipe):
+        Thread.__init__(self)
+        self.queue = []
+        self.isStopping = False
+        self.pipe = pipe
+        self.condition = Condition()
+
+    def run(self):
+        self.pipe.isDebug and self.pipe.log("Buffer running...", isDebug=True)
+        chars = ''
+        self.acquire()
+        while not self.isStopping or len(self.queue):
+            while not self.isStopping and not len(self.queue):
+                self.wait()
+            if len(self.queue):
+                chars = ''.join(self.queue)
+                self.queue = []
+                self.release()
+                self.output(chars)
+                self.acquire()
+        self.release()
+        self.pipe.isDebug and self.pipe.log("Buffer stopping...", isDebug=True)
+
+    def append(self, chars):
+        #self.pipe.isDebug and self.pipe.log("Appending %s chars to buffer: %s" % (len(chars), repr(chars)), isDebug=True)
+        self.acquire()
+        self.queue.append(chars)
+        self.notify()
+        self.release()
+
+    def acquire(self):
+        #self.pipe.isDebug and self.pipe.log("Acquiring buffer lock...", isDebug=True)
+        self.condition.acquire()
+
+    def wait(self):
+        #self.pipe.isDebug and self.pipe.log("Waiting for buffer notification...", isDebug=True)
+        self.condition.wait()
+
+    def notify(self):
+        #self.pipe.isDebug and self.pipe.log("Notifying buffer...", isDebug=True)
+        self.condition.notify()
+
+    def release(self):
+        #self.pipe.isDebug and self.pipe.log("Releasing buffer lock...", isDebug=True)
+        self.condition.release()
+
+    def output(self, chars):
+        logchars = chars[:25] + (chars[25:] and '..')
+        #self.pipe.isDebug and self.pipe.log("Outputting %s buffered chars: %s" % (len(chars), repr(logchars)), isDebug=True)
+        self.pipe.log("Outputting %s buffered chars: %s" % (len(chars), repr(logchars)), isDebug=True)
+        self.pipe.stdout.write(chars)
+        self.pipe.stdout.flush()
+        #time.sleep(0.01)  # Allow buffer to build up.
+
+    def stop(self):
+        #self.pipe.log("Buffer receiving stop signal...", isDebug=True)
+        self.acquire()
+        self.isStopping = True
+        self.notify()
+        self.release()
+        #self.pipe.log("Buffer received stop signal.", isDebug=True)
+    
+
 class PipeThread(Thread):
 
-    def __init__(self, name, stdin, stdout, reader, logger, rewriter=None, validPatterns=[]):
+    def __init__(self, name, stdin, stdout, readerClass, logger, rewriter=None, validPatterns=[], isDebug=False, abortStdoutMessage='', abortStderrMessage=''):
         Thread.__init__(self)
         self.name = name
         self.stdin = stdin
         self.stdout = stdout
-        self.reader = reader
+        self.readerClass = readerClass
         self.logger = logger
         self.rewriter = rewriter
         self.validPatterns = validPatterns
-        self.service = None
         self.error = None
+        self.isDebug = isDebug
+        self.abortStdoutMessage = abortStdoutMessage
+        self.abortStderrMessage = abortStderrMessage
+        self.writeBuffer = WriteBuffer(self)
+        self.lastMessageCondition = Condition()
 
     def run(self):
-        self.log('Running...')
+        self.log('Running...', isDebug=True)
+        if self.writeBuffer:
+            self.writeBuffer.start()
         try:
             while True:
-                self.log('Reading...')
-                message = self.reader(self.stdin, logger=self.logger).read()
-                self.log("Received message: '%s'" % message)
+                reader = self.readerClass(self.stdin, self)
+                # Read message from stdin.
+                message = reader.read()
+                #self.isDebug and self.log("Received message with %s chars." % len(message), isDebug=True)
+                if self.isDebug:
+                    logchars = repr(message)
+                    logchars = logchars[:60] + (logchars[60:] and '...')
+                    self.log("Received message with %s chars: %s" % (len(message), logchars), isDebug=True)
                 # Rewrite URLs.
                 if self.rewriter:
                     message = self.rewriter.rewrite(message)
@@ -493,35 +592,62 @@ class PipeThread(Thread):
                         msg = "Message is not allowed: %s" % (message)
                         self.log(msg)
                         raise AccessDenied, msg
-                self.stdout.write(message)
-                self.stdout.flush()
-                #self.log("Sent message: '%s'" % message)
+                if self.writeBuffer:
+                    if not self.writeBuffer.is_alive():
+                        raise Exception, "Buffer thread is no longer active."
+                    self.writeBuffer.append(message)
+                else:
+                    self.stdout.write(message)
+                    self.stdout.flush()
         except EndOfFile:
-            #self.log('End of file.')
-            self.stdout.close()
-            self.stdout = None
-            self.stdin = None
+            self.isDebug and self.log('End of file.', isDebug=True)
         except Exception, inst:
             self.error = inst
-            self.log('Error: %s' % inst)
-            self.stdout.write('\n')
-            self.stdout.flush()
+            self.log('Handling error: %s' % inst, isDebug=True)
+            if self.abortStdoutMessage:
+                self.isDebug and self.log('Writing abort message to stdout: %s' % repr(self.abortStdoutMessage), isDebug=True)
+                if self.writeBuffer:
+                    if self.writeBuffer.is_alive():
+                        self.writeBuffer.append(self.abortStdoutMessage)
+                else:
+                    self.stdout.write(self.abortMessage)
+                    self.stdout.flush()
+            if self.abortStderrMessage:
+                self.isDebug and self.log('Writing abort message to stderr: %s' % repr(self.abortStderrMessage), isDebug=True)
+                sys.stderr.write(self.abortStderrMessage)
+        finally:
+            if self.writeBuffer:
+                self.writeBuffer.stop()
+                self.isDebug and self.log('Waiting for write buffer to stop...', isDebug=True)
+                self.writeBuffer.join()
+            self.isDebug and self.log('Closing thread\'s stdout.', isDebug=True)
             self.stdout.close()
             self.stdout = None
             self.stdin = None
-            #raise
-        #finally:
-            #self.log('Stopping...')
+            self.isDebug and self.log('Stopping.', isDebug=True)
 
-    def log(self, line):
-        self.logger.info(self.name+": "+line)
+    def log(self, line, isDebug=False, isError=False):
+        line = "Pipe %s: %s" % (self.name, line)
+        if isDebug:
+            self.logger.debug(line)
+        elif isError:
+            self.logger.error(line)
+        else:
+            self.logger.info(line)
 
 
-class ReadSvnMessage(object):
+class BaseReader(object):
 
-    def __init__(self, stream, logger):
-        self.stream = stream
-        self.logger = logger
+    def __init__(self, stdin, client):
+        self.stdin = stdin
+        self.client = client
+
+    def log(self, line, isDebug=False, isError=False):
+        if hasattr(self.client, 'log'):
+            self.client.log(line, isDebug=isDebug, isError=isError)
+
+
+class ReadSvnMessage(BaseReader):
 
     def read(self):
         hasMessage = False
@@ -530,8 +656,8 @@ class ReadSvnMessage(object):
         token = ''
         while not hasMessage:
             hasToken = False
-            c = self.stream.read(1)
-            #self.logger.info("Char: '%s' brackets: %s token: %s" % (c, brackets, token))
+            c = self.stdin.read(1)
+            #self.log("Char: '%s' brackets: %s token: %s" % (c, brackets, token), isDebug=True)
             if c == '':
                 raise EndOfFile
             token += c
@@ -543,78 +669,80 @@ class ReadSvnMessage(object):
                 brackets -= 1
             elif re.match('\d+:', token):
                 octetCount = int(token.strip(':'))
-                token += self.stream.read(octetCount)
+                token += self.stdin.read(octetCount)
                 hasToken = True
             elif re.match('\S+\s', token):
                 hasToken = True
             elif re.match('\s', token):
                 hasToken = True
             if hasToken:
-                #self.logger.info("Token: '%s'" % token)
+                #self.log("Token: '%s'" % token, isDebug=True)
                 message += token
                 token = ''
                 if not brackets:
                     hasMessage = True
-                    #self.logger.info("Message: '%s'" % message)
+                    #self.log("Message: '%s'" % message, isDebug=True)
         return message
 
 
-class ReadHgMessage(object):
+class ReadHgMessage(BaseReader):
 
-    def __init__(self, stream, logger):
-        self.stream = stream
-        self.logger = logger
+    characterCountPtn = re.compile('(.*\s)?(\d+)\n$')
 
     def read(self):
         hasMessage = False
         message = ''
-        token = ''
         while not hasMessage:
+            token = ''
             hasToken = False
-            c = self.stream.read(1)
-            #self.logger.info("Char: '%s' token: %s" % (c, token))
-            if c == '':
-                self.logger.info("Raising EOF.")
-                raise EndOfFile
-            token += c
-            if token in ['between\n', 'changegroup\n', 'branches\n', 'changegroupsubset\n']:
-                # There is more message.
-                pass
-            elif token.endswith('\n'):
-                hasToken = True
-            if hasToken:
-                #self.logger.info("Token: '%s'" % token)
-                message += token
-                messageStripped = message.strip()
-                tokenSplit = token.strip().split()
-                if message in ['hello\n', 'between\n', 'heads\n']: #, 'unbundle\n']:
-                    # There is no more message.
+            while not hasToken:
+                char = self.stdin.read(1)
+                #self.log("New char: %s" % repr(char), isDebug=True)
+                if char == '':
+                    # That's the end of the file.
+                    if token or message:
+                        raise Exception, "Premature end of file: chars not formed into message: %s" % (message + token)
+                    else:
+                        self.log("Raising EOF.", isDebug=True)
+                        raise EndOfFile
+                token += char
+                if token in ['between\n', 'changegroup\n', 'branches\n', 'changegroupsubset\n', 'batch\n']:
+                    # Wait for the next new line.
                     pass
-                elif tokenSplit and re.match('\d+$', tokenSplit[-1]):
-                    # There is more message, get it now.
-                    length = tokenSplit[-1]
-                    message += self.stream.read(int(length))
-                else:
-                    raise AccessDenied, "KForge doesn't yet know about mercurial command: %s" % message
-                token = ''
+                elif token.endswith('\n'):
+                    # Let's call this a token.
+                    hasToken = True
+            #self.log("New token: '%s'" % token, isDebug=True)
+            message += token
+            if message in ['hello\n', 'between\n', 'heads\n', 'unbundle\n', 'branchmap\n', 'batch\n', 'listkeys\n']:
+                # Simple command message so we're done.
                 hasMessage = True
+            elif self.characterCountPtn.search(token):
+                # Read that amount of chars and we're done.
+                length = self.characterCountPtn.search(token).group(2)
+                length = int(length)
+                chars = self.stdin.read(length)
+                #self.log("New chars: %s" % (repr(chars)), isDebug=True)
+                message += chars
+                hasMessage = True
+            else:
+                # This is the inbound thread, so be defensive.
+                msg = "Unknown mercurial message: %s" % repr(message)
+                raise Exception, msg
+            token = ''
         return message
 
 
-class ReadSingleChar(object):
-
-    def __init__(self, stream, logger):
-        self.stream = stream
-        self.logger = logger
-
-    def read(self):
-        message = ''
-        c = self.stream.read(1)
-        if c == '':
-            raise EndOfFile
-        message = c
-        #self.logger.info("Single char message: '%s'" % (c))
-        return message
+#class ReadSingleChar(BaseReader):
+#
+#    def read(self):
+#        message = ''
+#        c = self.stdin.read(1)
+#        if c == '':
+#            raise EndOfFile
+#        message = c
+#        #self.log("Single char message: '%s'" % (c), isDebug=True)
+#        return message
 
 
 class UrlRewriter(object):
@@ -656,9 +784,8 @@ class UrlRewriter(object):
                 hasToken = True
             elif re.match('\d+:', token):
                 octetCount = int(token.strip(':'))
-                while octetCount:
-                    token += message.pop(0)
-                    octetCount -= 1
+                token += ''.join(message[:octetCount])
+                message = message[octetCount:]
                 hasToken = True
             elif re.match('\S+\s', token):
                 hasToken = True
