@@ -4,7 +4,8 @@ import kforge.command.initialisetest
 import kforge.command.projecttest
 import kforge.command.membertest
 import kforge.command.servicetest
-import kforge.command.emailpasswordtest
+import kforge.command.emailjoinrequesttest
+import kforge.command.emailmailmanpasswordtest
 #import kforge.command.projectwithadministratortest
 from kforge.exceptions import *
 
@@ -15,8 +16,9 @@ def suite():
         kforge.command.projecttest.suite(),
         kforge.command.membertest.suite(),
         kforge.command.servicetest.suite(),
+        kforge.command.emailjoinrequesttest.suite(),
+        kforge.command.emailmailmanpasswordtest.suite(),
 #        kforge.command.projectwithadministratortest.suite(),
-        kforge.command.emailpasswordtest.suite(),
     ]
     return unittest.TestSuite(suites)
 
