@@ -1,6 +1,8 @@
 import unittest
 import os
 import kforge.dictionary
+from kforge.dictionarywords import PROJECTS_PATH
+from kforge.dictionarywords import PLUGINS_AVAILABLE
 
 def suite():
     "Return a TestSuite of kforge.dom TestCases."
@@ -20,5 +22,6 @@ class SystemDictionaryTest(unittest.TestCase):
         self.failUnless(self.dictionary)
     
     def testGetValues(self):
-        self.failUnless(self.dictionary.has_key('project_data_dir'))
+        self.failUnless(self.dictionary.has_key(PROJECTS_PATH))
+        self.failUnless(self.dictionary.has_key(PLUGINS_AVAILABLE))
 
