@@ -1,20 +1,23 @@
 from kforge.django.apps.kui.views.projectHasMany import ProjectHasManyView
+from dm.view.base import AbstractPendingView
 from dm.view.base import AbstractListHasManyView
 from dm.view.base import AbstractCreateHasManyView
+from dm.view.base import AbstractApproveHasManyView
+from dm.view.base import AbstractRejectHasManyView
 from dm.view.base import AbstractReadHasManyView
 from dm.view.base import AbstractUpdateHasManyView
 from dm.view.base import AbstractDeleteHasManyView
-from kforge.django.apps.kui.views import manipulator
 import kforge.command
 
 from kforge.command.emailjoinrequest import EmailJoinApprove, EmailJoinReject
 
+
 class MemberView(ProjectHasManyView):
 
     hasManyClassName = 'Member'
 
-    def __init__(self, list_name='members', **kwds):
-        super(MemberView, self).__init__(hasManyName=list_name, **kwds)
+    def __init__(self, **kwds):
+        super(MemberView, self).__init__(hasManyName='members', **kwds)
 
     def setContext(self):
         super(MemberView, self).setContext()
@@ -23,7 +26,7 @@ class MemberView(ProjectHasManyView):
         })
 
     def setMinorNavigationItem(self):
-        self.minorNavigationItem = '/project/%s/members/' % self.domainObjectKey
+        self.minorNavigationItem = '/projects/%s/members/' % self.domainObjectKey
 
 
 class MemberListView(MemberView, AbstractListHasManyView):
@@ -42,30 +45,25 @@ class MemberCreateView(MemberView, AbstractCreateHasManyView):
         return self.canCreateMember()
 
     def makePostManipulateLocation(self):
-        return '/project/%s/%s/' % (
-            self.domainObjectKey, self.hasManyName
+        return '/projects/%s/' % (
+            self.domainObjectKey
         )
-
-    def getManipulatorClass(self):
-        return manipulator.MemberCreateManipulator
+        #return '/projects/%s/%s/' % (
+        #    self.domainObjectKey, self.hasManyName
+        #)
 
     def manipulateDomainObject(self):
-        #We want to make sure that if this member is pending, we remove them from that list
         super(MemberCreateView, self).manipulateDomainObject()
-        if not self.getFormErrors():
-            if not self.domainObject:
-                raise "View did not produce an object."
-            project = self.domainObject
-            form = self.getForm()
-            personName = form['person'].data
-            person = self.registry.persons[personName]
-            if person in project.pending_members:
-                memberCommand = kforge.command.PendingMemberDelete(
-                    projectName=project.name, personName=person.name
-                )
-                memberCommand.execute()
-                msg = "Deleted member %s from pending list for project %s since they were added as a member" % (person.name, project.name)
-                self.logger.debug(msg)
+        # Make sure that if this member is pending, we remove them from that list.
+        # Todo: Push this logic down into the model: on create, if pending
+        # register has object by key of created object, then delete pending object?
+        project = self.domainObject
+        personUri = self.getRequestParam('person')
+        person = self.registry.dereference(personUri).target
+        if person in project.members.pending:
+            project.members.pending[person].delete()
+            msg = "Deleted member '%s' from pending list for project %s since they were added as a member" % (person.name, project.name)
+            self.logger.debug(msg)
 
 
 class MemberUpdateView(MemberView, AbstractUpdateHasManyView):
@@ -76,12 +74,12 @@ class MemberUpdateView(MemberView, AbstractUpdateHasManyView):
         return self.canUpdateMember()
 
     def makePostManipulateLocation(self):
-        return '/project/%s/%s/' % (
-            self.domainObjectKey, self.hasManyName
+        return '/projects/%s/' % (
+            self.domainObjectKey
         )
-
-    def getManipulatorClass(self):
-        return manipulator.MemberUpdateManipulator
+        #return '/projects/%s/%s/' % (
+        #    self.domainObjectKey, self.hasManyName
+        #)
 
 
 class MemberDeleteView(MemberView, AbstractDeleteHasManyView):
@@ -94,69 +92,56 @@ class MemberDeleteView(MemberView, AbstractDeleteHasManyView):
 
     def makePostManipulateLocation(self):
         if self.session.person == self.member.person:
-            return '/person/home/'
+            return '/people/%s/' % self.session.person.name
         else:
-            return '/project/%s/%s/' % (
-                self.domainObjectKey, self.hasManyName
+            return '/projects/%s/' % (
+                self.domainObjectKey
             )
+            #return '/projects/%s/%s/' % (
+            #    self.domainObjectKey, self.hasManyName
+            #)
 
 
-class MemberRejectView(MemberView, AbstractDeleteHasManyView):
+class MemberApproveView(MemberView, AbstractApproveHasManyView):
+
+    templatePath = 'member/approve'
+    fieldNames = ['role']
+    
+    def canAccess(self):
+        return self.canCreateMember()
+
+    def manipulateDomainObject(self):
+        member = self.getManipulatedDomainObject()
+        super(MemberApproveView, self).manipulateDomainObject()
+        # Send e-mail to the member letting them know they got approved.
+        emailCommand = EmailJoinApprove(member.project, member.person)
+        emailCommand.execute()
+
+    def makePostManipulateLocation(self):
+        return '/projects/%s/members/' % (
+            self.domainObjectKey
+        )
+
+
+class MemberRejectView(MemberView, AbstractRejectHasManyView):
 
     templatePath = 'member/reject'
     
     def canAccess(self):
         return self.canCreateMember()
 
-    def makePostManipulateLocation(self):
-        return '/project/%s/members/' % (
-            self.domainObjectKey
-        )
-
     def manipulateDomainObject(self):
+        member = self.getManipulatedDomainObject()
         super(MemberRejectView, self).manipulateDomainObject()
-        #we'll also send an e-mail to the member letting them know they got rejected
-        project = self.domainObject
-        person = self.registry.persons[self.hasManyKey]
-        emailCommand = EmailJoinReject(project, person)
+        # Send an e-mail to the member letting them know they got rejected.
+        emailCommand = EmailJoinReject(member.project, member.person)
         emailCommand.execute()
 
-class MemberApproveView(MemberView, AbstractDeleteHasManyView):
-
-    templatePath = 'member/approve'
-    
-    def canAccess(self):
-        return self.canCreateMember()
-
     def makePostManipulateLocation(self):
-        return '/project/%s/members/' % (
+        return '/projects/%s/members/' % (
             self.domainObjectKey
         )
 
-    def getManipulatorClass(self):
-        return manipulator.MemberUpdateManipulator
-
-    def manipulateDomainObject(self):
-        super(MemberApproveView, self).manipulateDomainObject()
-        if not self.getFormErrors():
-            if not self.domainObject:
-                raise "View did not produce an object."
-            form = self.getForm()
-            roleName = form['role'].data
-            role = self.registry.roles[roleName]
-            project = self.domainObject
-            person = self.registry.persons[self.hasManyKey]
-            memberCommand = kforge.command.MemberCreate(
-                project=project, person=person, role=role
-            )
-            memberCommand.execute()
-            msg = "Approved member %s for project %s" % (person.name, project.name)
-            self.logger.debug(msg)
-
-            #we'll also send an e-mail to the member letting them know they got approved
-            emailCommand = EmailJoinApprove(project, person)
-            emailCommand.execute()
-
 
 def list(request, projectName=''):
     view = MemberListView(
@@ -190,7 +175,6 @@ def delete(request, projectName='', personName=''):
 
 def approve(request, projectName='', personName=''):
     view = MemberApproveView(
-        list_name='pending_members',
         request=request,
         domainObjectKey=projectName,
         hasManyKey=personName,
@@ -199,7 +183,6 @@ def approve(request, projectName='', personName=''):
 
 def reject(request, projectName='', personName=''):
     view = MemberRejectView(
-        list_name='pending_members',
         request=request,
         domainObjectKey=projectName,
         hasManyKey=personName,
