@@ -1,4 +1,4 @@
-from dm.view.base import *
+from dm.view.base import AbstractHasManyView
 from kforge.django.apps.kui.views.project import ProjectInstanceView
 import kforge.command
 from kforge.exceptions import KforgeCommandError
