@@ -1,123 +1,70 @@
 import unittest
 from kforge.django.apps.kui.views.testunit import TestCase
 from dm import webkit
-if webkit.webkitName == 'django' and webkit.webkitVersion == '1.0':
-    from kforge.django.apps.kui.views.manipulator import PasswordField
-    from kforge.django.apps.kui.views.manipulator import ProjectNameField
-    from kforge.django.apps.kui.views.manipulator import PersonNameField
-    from kforge.django.apps.kui.views.manipulator import ServiceNameField
+#if webkit.webkitName == 'django' and webkit.webkitVersion == '1.0':
+#    from kforge.django.apps.kui.views.manipulator import PasswordField
+#    from kforge.django.apps.kui.views.manipulator import ProjectNameField
+#    from kforge.django.apps.kui.views.manipulator import PersonNameField
+#    from kforge.django.apps.kui.views.manipulator import ServiceNameField
 from kforge.django.apps.kui.views.manipulator import PersonCreateManipulator
 from kforge.django.apps.kui.views.manipulator import PersonUpdateManipulator
-from kforge.django.apps.kui.views.manipulator import ProjectCreateManipulator
-from kforge.django.apps.kui.views.manipulator import ProjectUpdateManipulator
-from kforge.django.apps.kui.views.manipulator import ServiceCreateManipulator
-from kforge.django.apps.kui.views.manipulator import ServiceUpdateManipulator
 from kforge.django.apps.kui.views.manipulator import DomainObjectManipulator
 from kforge.django.apps.kui.views.manipulator import HasManyManipulator
 from django.utils.datastructures import MultiValueDict
 from kforge.ioc import *
+from dm.dom.pickers import *
+from dm.exceptions import *
 
 def suite():
     suites = [
-        unittest.makeSuite(TestPersonCreateManipulator),
-        unittest.makeSuite(TestPersonUpdateManipulator),
-        unittest.makeSuite(TestProjectCreateManipulator),
-        unittest.makeSuite(TestProjectUpdateManipulator),
-        unittest.makeSuite(TestServiceCreateManipulator),
-        unittest.makeSuite(TestServiceUpdateManipulator),
+    #    unittest.makeSuite(TestPersonCreateManipulator),
+    #    unittest.makeSuite(TestPersonUpdateManipulator),
+    #    unittest.makeSuite(TestProjectCreateManipulator),
+    #    unittest.makeSuite(TestProjectUpdateManipulator),
         # Todo: Move these to dm.
-        unittest.makeSuite(TestDomainObjectManipulator),
+        unittest.makeSuite(TestDomainObjectManipulatorValidate),
         unittest.makeSuite(TestDomainObjectManipulatorCreate),
         unittest.makeSuite(TestDomainObjectManipulatorUpdate),
         unittest.makeSuite(TestHasManyManipulator),
     ]
-    if webkit.webkitName == 'django' and webkit.webkitVersion == '1.0':
-        suites = [
-            unittest.makeSuite(TestPasswordField),
-            unittest.makeSuite(TestPersonNameField),
-            unittest.makeSuite(TestProjectNameField),
-            unittest.makeSuite(TestServiceNameField),
-        ] + suites
-    suites = [
-        unittest.makeSuite(TestDomainObjectManipulator),
-        unittest.makeSuite(TestHasManyManipulator),
-    ]
+    #suites = [
+    #    unittest.makeSuite(TestDomainObjectManipulator),
+    #    unittest.makeSuite(TestHasManyManipulator),
+    #]
     return unittest.TestSuite(suites)
 
 
-if webkit.webkitName == 'django' and webkit.webkitVersion == '1.0':
-
-    class FieldTestCase(TestCase):
-
-        fieldClass = None
-        fieldValids = []
-        fieldInvalids = []
-
-        def setUp(self):
-            self.field = self.fieldClass()
-
-        def tearDown(self):
-            self.field = None
-
-        def testField(self):
-            for v in self.fieldValids:
-                try:
-                    self.field.clean(v)
-                except webkit.ValidationError, inst:
-                    self.fail("Should be valid: %s %s %s" % (v, self.field, inst)) 
-            for v in self.fieldInvalids:
-                try:
-                    self.field.clean(v)
-                except webkit.ValidationError:
-                    pass
-                else:
-                    self.fail("Should be invalid: %s %s" % (v, self.field)) 
-
-
-    class TestPasswordField(FieldTestCase):
-        
-        fieldClass = PasswordField
-        fieldValids = ['meme']
-        fieldInvalids = ['', 'm', 'me', 'mem']
-       
-
-    class TestPersonNameField(FieldTestCase):
-        
-        fieldClass = PersonNameField
-        fieldValids = ['..', 'jo', 'joh', 'john', 'john99', 'johnbywater', 'john-bywater', 'john.bywater', 'john_bywater', 'johnbywaterrrrrrrrrr']
-        fieldInvalids = ['', '.', '', 'm', '1', ' ', 'home', 'create', 'find', 'search', 'update', 'delete', 'purge', 'login', 'logout', 'johnbywaterrrrrrrrrrrz']
-
-       
-    class TestProjectNameField(FieldTestCase):
-        
-        fieldClass = ProjectNameField
-        fieldValids = ['mem', 'meme', 'mem99' ]
-        fieldInvalids = ['', 'm', 'me', 'meme.', 'meme-', 'meme_', 'home' ,'create' ,'find' ,'search' ,'update' ,'delete' ,'purge' ,'admin' ,'person' ,'project' ,'login' ,'media', 'sixteencharsssss']
-
-
-    class TestServiceNameField(FieldTestCase):
-        
-        fieldClass = ServiceNameField
-        fieldValids = ['m', 'me', 'mem', 'meme', 'mem99', 'Mem']
-        fieldInvalids = ['', '%', '$', '/', '&', 'seventeencharssss']
-
-
 class ManipulatorTestCase(TestCase):
 
     def setUp(self):
         super(ManipulatorTestCase, self).setUp()
-        self.manipulator = self.buildManipulator()
+        self.manipulator = self.createManipulator()
 
     def tearDown(self):
         self.manipulator = None
         super(ManipulatorTestCase, self).tearDown()
     
-    def test_exists(self):
+    def checkFormField(self, name, valids, invalids):
+        field = self.manipulator.fields[name]
+        for v in valids:
+            try:
+                field.clean(v)
+            except ValidationError, inst:
+                self.fail("Should be valid: %s %s %s" % (v, self.field, inst)) 
+        for v in invalids:
+            try:
+                field.clean(v)
+            except ValidationError:
+                pass
+            else:
+                self.fail("Should be invalid: %s %s" % (v, field))
+
+    def testExists(self):
         self.failUnless(self.manipulator)
         self.failUnless(self.manipulator.fields)
     
-    def buildManipulator(self):
-        raise "Abstract method not implemented."
+    def createManipulator(self):
+        raise Exception, "Abstract method not implemented."
 
 
 class PersonManipulatorTestCase(ManipulatorTestCase):
@@ -137,8 +84,8 @@ class PersonManipulatorTestCase(ManipulatorTestCase):
         self.tearDownFixtures()
         
     def tearDownFixtures(self):
-        if self.fixtureName in self.registry.persons.getAll():
-            person = self.registry.persons.getAll()[self.fixtureName]
+        if self.fixtureName in self.registry.people.getAll():
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
 
@@ -149,12 +96,18 @@ class TestPersonCreateManipulator(PersonManipulatorTestCase):
         super(TestPersonCreateManipulator, self).setUp()
         self.manipulator.create(self.validData)
 
+    def createManipulator(self):
+        return PersonCreateManipulator(objectRegister=self.registry.people, pickerClass=GetEditableAttributes)
+
     def testCreate(self):
-        person = self.registry.persons[self.fixtureName]
+        person = self.registry.people[self.fixtureName]
         self.failUnlessEqual(person.name, self.validData['name'])
     
-    def buildManipulator(self):
-        return PersonCreateManipulator(objectRegister=self.registry.persons)
+    def testFormFields(self):
+        self.checkFormField('name',
+            valids = ['..', 'jo', 'joh', 'john', 'john99', 'johnbywater', 'john-bywater', 'john.bywater', 'john_bywater', 'johnbywaterrrrrrrrrr'],
+            invalids = ['', '.', '', 'm', '1', ' ', 'home', 'create', 'find', 'search', 'update', 'delete', 'purge', 'login', 'logout', 'johnbywaterrrrrrrrrrrz'],
+        )
 
 
 class TestPersonUpdateManipulator(PersonManipulatorTestCase):
@@ -164,15 +117,15 @@ class TestPersonUpdateManipulator(PersonManipulatorTestCase):
         self.manipulator.create(self.validData)
 
     def testUpdate(self):
-        person = self.registry.persons[self.fixtureName]
+        person = self.registry.people[self.fixtureName]
         self.failUnlessEqual(person.name, self.validData['name'])
         self.validData['email'] = 'john@doe.com'
         self.manipulator.update(self.validData)
-        person = self.registry.persons[self.fixtureName]
+        person = self.registry.people[self.fixtureName]
         self.failUnlessEqual(person.email, self.validData['email'])
 
-    def buildManipulator(self):
-        return PersonUpdateManipulator(objectRegister=self.registry.persons)
+    def createManipulator(self):
+        return PersonUpdateManipulator(objectRegister=self.registry.people, pickerClass=GetEditableAttributes)
 
 
 class ProjectManipulatorTestCase(ManipulatorTestCase):
@@ -210,8 +163,14 @@ class TestProjectCreateManipulator(ProjectManipulatorTestCase):
         project = self.registry.projects[self.fixtureName]
         self.failUnlessEqual(project.name, self.validData['name'])
     
-    def buildManipulator(self):
-        return ProjectCreateManipulator(objectRegister=self.registry.projects)
+    def createManipulator(self):
+        return DomainObjectManipulator(objectRegister=self.registry.projects, pickerClass=GetInitableAttributes)
+
+    def testFormFields(self):
+        self.checkFormField('name',
+            valids = ['mem', 'meme', 'mem99' ],
+            invalids = ['', 'm', 'me', 'meme.', 'meme-', 'meme_', 'home' ,'create' ,'find' ,'search' ,'update' ,'delete' ,'purge' ,'admin' ,'person' ,'project' ,'login' ,'media', 'sixteencharsssss'],
+        )
 
 
 class TestProjectUpdateManipulator(ProjectManipulatorTestCase):
@@ -228,8 +187,8 @@ class TestProjectUpdateManipulator(ProjectManipulatorTestCase):
         project = self.registry.projects[self.fixtureName]
         self.failUnlessEqual(project.description, self.validData['description'])
 
-    def buildManipulator(self):
-        return ProjectUpdateManipulator(objectRegister=self.registry.projects)
+    def createManipulator(self):
+        return DomainObjectManipulator(objectRegister=self.registry.projects, pickerClass=GetEditableAttributes)
 
 
 class ServiceManipulatorTestCase(ManipulatorTestCase):
@@ -265,130 +224,143 @@ class ServiceManipulatorTestCase(ManipulatorTestCase):
             project.purge()
 
 
-class TestServiceCreateManipulator(ServiceManipulatorTestCase):
-
-    def setUp(self):
-        super(TestServiceCreateManipulator, self).setUp()
-        self.manipulator.create(self.validData)
-
-    def testCreate(self):
-        service = self.projectFixture.services[self.fixtureName]
-        self.failUnlessEqual(service.name, self.validData['name'])
-    
-    def buildManipulator(self):
-        return ServiceCreateManipulator(objectRegister=self.registry.services)
-
-
-class TestServiceUpdateManipulator(ServiceManipulatorTestCase):
-
-    def setUp(self):
-        super(TestServiceUpdateManipulator, self).setUp()
-        self.manipulator.create(self.validData)
-
-    def testUpdate(self):
-        service = self.projectFixture.services[self.fixtureName]
-        self.failUnlessEqual(service.name, self.validData['name'])
-        newName = 'Blah blah blah'
-        self.validData['name'] = newName
-        self.manipulator.update(self.validData)
-        service = self.projectFixture.services[newName]
-        self.failUnlessEqual(service.name, newName) 
-        self.failUnlessEqual(service.plugin.name, self.validData['plugin'])
-
-    def buildManipulator(self):
-        return ServiceUpdateManipulator(objectRegister=self.registry.services)
+# Todo: Put service name validation checks somewhere else.
 
+#class TestServiceCreateManipulator(ServiceManipulatorTestCase):
+#
+#    def setUp(self):
+#        super(TestServiceCreateManipulator, self).setUp()
+#        self.manipulator.create(self.validData)
+#
+#    def testCreate(self):
+#        service = self.projectFixture.services[self.fixtureName]
+#        self.failUnlessEqual(service.name, self.validData['name'])
+#    
+#    def createManipulator(self):
+#        return ServiceCreateManipulator(objectRegister=self.registry.services, pickerClass=GetInitableAttributes)
+#
+#    def testFormFields(self):
+#        self.checkFormField('name',
+#            valids = ['m', 'me', 'mem', 'meme', 'mem99', 'Mem'],
+#            invalids = ['', '%', '$', '/', '&', 'seventeencharssss'],
+#        )
+#
+#
+#class TestServiceUpdateManipulator(ServiceManipulatorTestCase):
+#
+#    def setUp(self):
+#        super(TestServiceUpdateManipulator, self).setUp()
+#        self.manipulator.create(self.validData)
+#
+#    def testUpdate(self):
+#        service = self.projectFixture.services[self.fixtureName]
+#        self.failUnlessEqual(service.name, self.validData['name'])
+#        newName = 'Blah blah blah'
+#        self.validData['name'] = newName
+#        self.manipulator.update(self.validData)
+#        service = self.projectFixture.services[newName]
+#        self.failUnlessEqual(service.name, newName) 
+#        self.failUnlessEqual(service.plugin.name, self.validData['plugin'])
+#
+#    def createManipulator(self):
+#        return ServiceUpdateManipulator(objectRegister=self.registry.services, pickerClass=GetEditableAttributes)
+#
 
 class DomainObjectManipulatorTestCase(ManipulatorTestCase):
 
-    fixtureName = 'DomainObjectManipulatorTestCase'
+    fixtureName = 'ManipulatorTestCase'
 
     def setUp(self):
         super(DomainObjectManipulatorTestCase, self).setUp()
         self.validData = MultiValueDict()
         self.validData['name'] = self.fixtureName
-        self.validData['fullname'] = 'DomainObject Manipulator TestCase'
+        self.validData['fullname'] = '' # Not required.
         self.validData['password'] = 'password'
         self.validData['email'] = 'email@email.com'
-        self.validData['role'] = 'Visitor'
-        self.validData['state'] = 'active'
+        self.validData['role'] = '/roles/Visitor'
+        #self.validData['state'] = 'active'
         self.validData.setlist('memberships', ['administration', 'example'])
         self.invalidData = MultiValueDict()
-        self.invalidData['name'] = self.fixtureName
-        self.invalidData['fullname'] = 'DomainObject Manipulator TestCase'
+        self.invalidData['name'] = ''
         self.invalidData['password'] = 'password'
         self.invalidData['email'] = 'email@email.com'
-        self.invalidData['role'] = 'Destroyer'
-        self.invalidData['state'] = 'active'
 
     def tearDown(self):
         super(DomainObjectManipulatorTestCase, self).tearDown()
-        if self.fixtureName in self.registry.persons.getAll():
-            person = self.registry.persons.getAll()[self.fixtureName]
+        if self.fixtureName in self.registry.people.getAll():
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
 
-    def buildManipulator(self):
-        return DomainObjectManipulator(objectRegister=self.registry.persons)
+    def createManipulator(self):
+        return DomainObjectManipulator(objectRegister=self.registry.people, pickerClass=self.pickerClass)
 
 
-class TestDomainObjectManipulator(DomainObjectManipulatorTestCase):
+class TestDomainObjectManipulatorValidate(DomainObjectManipulatorTestCase):
 
-    def testGetValidationErrors(self):
-        errors = self.manipulator.getValidationErrors(self.validData)
-        self.failIf(errors, str(errors))
-        errors = self.manipulator.getValidationErrors(self.invalidData)
-        self.failUnless(errors, str(errors))
+    pickerClass = GetInitableAttributes
 
-    def testDecodeHtml(self):
-        self.manipulator.decodeHtml(self.validData)
+    def testValidate(self):
+        self.manipulator.validate(self.validData)
+        self.failUnlessRaises(FormError, self.manipulator.validate, self.invalidData)
+        self.failUnless(self.manipulator.getErrors())
+        self.failUnless(self.manipulator.getErrors()['name'])
 
 
 class TestDomainObjectManipulatorCreate(DomainObjectManipulatorTestCase):
 
-    def setUp(self):
-        super(TestDomainObjectManipulatorCreate, self).setUp()
-        self.manipulator.create(self.validData)
+    pickerClass = GetInitableAttributes
 
     def testCreate(self):
-        person = self.registry.persons[self.fixtureName]
+        self.manipulator.create(self.validData)
+        person = self.registry.people[self.fixtureName]
         self.failUnlessEqual(person.name, self.validData['name'])
         self.failUnlessEqual(person.fullname, self.validData['fullname'])
-        self.failUnlessEqual(person.email, self.validData['email'])
-        self.failUnlessEqual(person.role.getRegisterKeyValue(), self.validData['role'])
-        adminProject = self.registry.projects['administration']
-        self.failUnless(adminProject in person.memberships)
+        #self.failUnlessEqual(person.email, self.validData['email'])
+        self.failUnlessEqual(person.role.getUri(), self.validData['role'])
+        self.failUnlessRaises(FormError, self.manipulator.create, self.invalidData)
 
 
 class TestDomainObjectManipulatorUpdate(DomainObjectManipulatorTestCase):
 
+    pickerClass = GetAdminEditableAttributes
+
     def setUp(self):
         super(TestDomainObjectManipulatorUpdate, self).setUp()
-        self.manipulator.create(self.validData)
-        objectRegister = self.registry.persons
-        domainObject = self.manipulator.domainObject
-        self.manipulator = None
+        objectRegister = self.registry.people
+        manipulator = DomainObjectManipulator(objectRegister=objectRegister, pickerClass=GetInitableAttributes)
+        manipulator.create(self.validData)
+        domainObject = manipulator.domainObject
         self.manipulator = DomainObjectManipulator(
             objectRegister=objectRegister,
             domainObject=domainObject,
+            pickerClass=self.pickerClass
         )
+
+    def tearDown(self):
+        super(TestDomainObjectManipulatorUpdate, self).tearDown()
+        personName = self.validData['name']
+        if personName in self.registry.people:
+            person = self.registry.people[personName]
+            person.delete()
+            person.purge()
         
     def testUpdateString(self):
-        self.validData['fullname'] = 'Update ' + self.validData['fullname']
+        self.validData['fullname'] = 'Test Fullname'
         self.manipulator.update(self.validData)
-        person = self.registry.persons[self.fixtureName]
+        person = self.registry.people[self.fixtureName]
         self.failUnlessEqual(person.fullname, self.validData['fullname'])
 
     def testUpdateHasA(self):
-        self.validData['role'] = 'Developer'
+        self.validData['role'] = '/roles/Developer'
         self.manipulator.update(self.validData)
-        person = self.registry.persons[self.fixtureName]
-        self.failUnlessEqual(person.role.getRegisterKeyValue(), self.validData['role'])
+        person = self.registry.people[self.fixtureName]
+        self.failUnlessEqual(person.role.getUri(), self.validData['role'])
 
     def testUpdateHasMany(self):
         self.validData.setlist('memberships', ['example'])
         self.manipulator.update(self.validData)
-        person = self.registry.persons[self.fixtureName]
+        person = self.registry.people[self.fixtureName]
         adminProject = self.registry.projects['administration']
         self.failIf(adminProject in person.memberships)
 
@@ -401,13 +373,13 @@ class HasManyManipulatorTestCase(ManipulatorTestCase):
 
     def setUp(self):
         super(HasManyManipulatorTestCase, self).setUp()
-        self.person = self.registry.persons[self.personName]
+        self.person = self.registry.people[self.personName]
         self.validData = MultiValueDict()
         self.validData['person'] = self.personName
-        self.validData['role'] = self.roleName
+        self.validData['role'] = '/roles/%s' % self.roleName
         self.invalidData = MultiValueDict()
         self.invalidData['person'] = self.personName
-        self.invalidData['role'] = 'Destroyer'
+        self.invalidData['role'] = '/roles/Destroyer'
 
     def tearDown(self):
         super(HasManyManipulatorTestCase, self).tearDown()
@@ -416,19 +388,17 @@ class HasManyManipulatorTestCase(ManipulatorTestCase):
             membership.delete()
             membership.purge()
 
-    def buildManipulator(self):
+    def createManipulator(self):
         self.project = self.registry.projects[self.projectName]
-        return HasManyManipulator(objectRegister=self.project.members)
+        return HasManyManipulator(objectRegister=self.project.members, pickerClass=GetEditableAttributes)
 
 
 class TestHasManyManipulator(HasManyManipulatorTestCase):
 
-    def testGetValidationErrors(self):
-        errors = self.manipulator.getValidationErrors(self.validData)
-        self.failIf(errors, str(errors))
-        errors = self.manipulator.getValidationErrors(self.invalidData)
-        self.failUnless(errors, str(errors))
-
-    def testDecodeHtml(self):
-        self.manipulator.decodeHtml(self.validData)
+    def testValidate(self):
+        try:
+            self.manipulator.validate(self.validData)
+        except FormError, inst:
+            self.fail("There were form errors: %s" % inst.errors)
+        self.failUnlessRaises(FormError, self.manipulator.validate, self.invalidData)
 
