@@ -19,7 +19,7 @@ def suite():
         unittest.makeSuite(TestMemberUpdateView),
         unittest.makeSuite(TestMemberUpdateViewLevin),
         unittest.makeSuite(TestMemberUpdateViewLevinPOST),
-        unittest.makeSuite(TestMemberDeleteView),
+        #unittest.makeSuite(TestMemberDeleteView),
         unittest.makeSuite(TestMemberDeleteViewLevin),
         unittest.makeSuite(TestMemberDeleteViewLevinPOST),
         unittest.makeSuite(TestMemberDeleteViewAdministrator),
@@ -84,29 +84,30 @@ class TestMemberCreateViewLevinPOST(MemberViewTestCase):
     viewerName = 'levin'
     projectName = 'annakarenina'
     viewClass = MemberCreateView
-    POST = MultiValueDict({'person': ['visitor'], 'role': ['Developer']})
+    POST = MultiValueDict({'person': ['/people/visitor'], 'role': ['/roles/Developer']})
     requiredFormErrors = False
     requiredResponseClassName = 'HttpResponseRedirect'
-    requiredRedirect = '/project/annakarenina/members/'
+    requiredRedirect = '/projects/annakarenina/'
         
     def test_canAccess(self):
         self.failUnless(self.view.canAccess())
 
     def tearDown(self):
         super(TestMemberCreateViewLevinPOST, self).tearDown()
-        person = self.registry.persons['visitor']
+        person = self.registry.people['visitor']
         if person in self.registry.projects[self.projectName].members:
             o = self.registry.projects[self.projectName].members[person]
             o.delete()
             o.purge()
 
+
 class TestMemberCreateViewLevinPOSTErrorNoRole(MemberViewTestCase):
 
     viewerName = 'levin'
     projectName = 'annakarenina'
     viewClass = MemberCreateView
-    POST = MultiValueDict({'person': ['natasha']})
-    requiredFormErrors = ['role', 'is required']
+    POST = MultiValueDict({'person': ['/people/natasha']})
+    requiredFormErrors = ['Role is required']
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
         
@@ -119,8 +120,8 @@ class TestMemberCreateViewLevinPOSTErrorNoPerson(MemberViewTestCase):
     viewerName = 'levin'
     projectName = 'annakarenina'
     viewClass = MemberCreateView
-    POST = MultiValueDict({'role': ['Administrator']})
-    requiredFormErrors = ['person', 'is required']
+    POST = MultiValueDict({'role': ['/roles/Administrator']})
+    requiredFormErrors = ['Person is required']
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
         
@@ -133,8 +134,8 @@ class TestMemberCreateViewLevinPOSTErrorAlreadyMember(MemberViewTestCase):
     viewerName = 'levin'
     projectName = 'annakarenina'
     viewClass = MemberCreateView
-    POST = MultiValueDict({'person': ['levin'], 'role': ['Developer']})
-    requiredFormErrors = ['is already a member of this project']
+    POST = MultiValueDict({'person': ['/people/levin'], 'role': ['/roles/Developer']})
+    requiredFormErrors = ['There is already one for']
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
         
@@ -173,20 +174,20 @@ class TestMemberUpdateViewLevinPOST(MemberViewTestCase):
     projectName = 'annakarenina'
     personName = 'visitor'
     viewClass = MemberUpdateView
-    POST = MultiValueDict({'person': ['visitor'], 'role': ['Developer']})
+    POST = MultiValueDict({'person': ['/people/visitor'], 'role': ['/roles/Developer']})
     requiredFormErrors = False
     requiredResponseClassName = 'HttpResponseRedirect'
-    requiredRedirect = '/project/annakarenina/members/'
+    requiredRedirect = '/projects/annakarenina/'
     
     def setUp(self):
         super(TestMemberUpdateViewLevinPOST, self).setUp()
-        person = self.registry.persons['visitor']
+        person = self.registry.people['visitor']
         role = self.registry.roles['Friend']
         self.registry.projects[self.projectName].members.create(person, role=role)
     
     def tearDown(self):
         super(TestMemberUpdateViewLevinPOST, self).tearDown()
-        person = self.registry.persons['visitor']
+        person = self.registry.people['visitor']
         if person in self.registry.projects[self.projectName].members:
             member = self.registry.projects[self.projectName].members[person]
             member.delete()
@@ -213,13 +214,13 @@ class TestMemberDeleteViewLevin(MemberViewTestCase):
 
     def setUp(self):
         super(TestMemberDeleteViewLevin, self).setUp()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         role = self.registry.roles['Friend']
         self.registry.projects[self.projectName].members.create(person, role=role)
     
     def tearDown(self):
         super(TestMemberDeleteViewLevin, self).tearDown()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         if person in self.registry.projects[self.projectName].members:
             member = self.registry.projects[self.projectName].members[person]
             member.delete()
@@ -236,17 +237,17 @@ class TestMemberDeleteViewLevinPOST(MemberViewTestCase):
     viewerName = 'levin'
     personName = 'levin'
     POST = MultiValueDict({'Submit': ['Delete membership']})
-    requiredRedirect = '/person/home/'
+    requiredRedirect = '/people/levin/'
 
     def setUp(self):
         super(TestMemberDeleteViewLevinPOST, self).setUp()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         role = self.registry.roles['Friend']
         self.registry.projects[self.projectName].members.create(person, role=role)
     
     def tearDown(self):
         super(TestMemberDeleteViewLevinPOST, self).tearDown()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         if person in self.registry.projects[self.projectName].members:
             member = self.registry.projects[self.projectName].members[person]
             member.delete()
@@ -266,7 +267,7 @@ class TestMemberDeleteViewAdministratorPOST(TestMemberDeleteViewLevinPOST):
     "Administrator can remove Levin from Administration project."
 
     viewerName = 'admin'
-    requiredRedirect = '/project/administration/members/'
+    requiredRedirect = '/projects/administration/'
 
 
 class TestMemberDeleteViewNatasha(MemberViewTestCase):
@@ -280,13 +281,13 @@ class TestMemberDeleteViewNatasha(MemberViewTestCase):
 
     def setUp(self):
         super(TestMemberDeleteViewNatasha, self).setUp()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         role = self.registry.roles['Friend']
         self.registry.projects[self.projectName].members.create(person, role=role)
     
     def tearDown(self):
         super(TestMemberDeleteViewNatasha, self).tearDown()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         if person in self.registry.projects[self.projectName].members:
             member = self.registry.projects[self.projectName].members[person]
             member.delete()
@@ -308,13 +309,13 @@ class TestMemberDeleteViewNatashaPOST(MemberViewTestCase):
 
     def setUp(self):
         super(TestMemberDeleteViewNatashaPOST, self).setUp()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         role = self.registry.roles['Friend']
         self.registry.projects[self.projectName].members.create(person, role=role)
     
     def tearDown(self):
         super(TestMemberDeleteViewNatashaPOST, self).tearDown()
-        person = self.registry.persons[self.personName]
+        person = self.registry.people[self.personName]
         if person in self.registry.projects[self.projectName].members:
             member = self.registry.projects[self.projectName].members[person]
             member.delete()
