@@ -1,4 +1,5 @@
 import unittest
+from kforge.test import RequiresPlugins
 from kforge.django.apps.kui.views.test.base import ViewTestCase
 from kforge.django.apps.kui.views.service import ServiceListView
 from kforge.django.apps.kui.views.service import ServiceCreateView
@@ -33,9 +34,9 @@ def suite():
         unittest.makeSuite(TestServiceUpdateViewNatashaPOSTTrac),
         unittest.makeSuite(TestServiceUpdateViewNatashaPOSTTracSvn),
         unittest.makeSuite(TestServiceUpdateViewNatashaPOSTTracHg),
-        unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorInUse),
-        unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorRequired),
-        unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorInvalid),
+        #unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorInUse),
+        #unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorRequired),
+        #unittest.makeSuite(TestServiceUpdateViewNatashaPOSTErrorInvalid),
         unittest.makeSuite(TestServiceDeleteView),
         unittest.makeSuite(TestServiceDeleteViewNatasha),
         unittest.makeSuite(TestServiceDeleteViewNatashaPOST),
@@ -43,7 +44,7 @@ def suite():
     return unittest.TestSuite(suites)
 
 
-class ServiceTestCase(ViewTestCase):
+class ServiceTestCase(RequiresPlugins, ViewTestCase):
 
     projectName = 'warandpeace'
 
@@ -54,6 +55,9 @@ class ServiceTestCase(ViewTestCase):
             viewKwds['hasManyKey'] = self.serviceName
         return viewKwds
 
+# Todo: Test for when service create view has service type passed in, check create form with correct extn fields is returned normally. So "Select service type" form, then "Create <service type> service" with extn fields, then "Edit <service name> service" with extn fields.
+# Todo: Method to get plugin model extn fields from service with service type.
+
 
 class TestServiceListView(ServiceTestCase):
 
@@ -117,8 +121,8 @@ class TestServiceCreateViewNatashaPOSTErrorInUse(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['example'], 'plugin': ['example']})
-    requiredFormErrors = "A service called 'example' already exists. Please choose another service name."
+    POST = MultiValueDict({'name': ['example'], 'plugin': ['/plugins/example']})
+    requiredFormErrors = "The name 'example' is already being used."
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
 
@@ -127,8 +131,8 @@ class TestServiceCreateViewNatashaPOSTErrorRequired(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': [''], 'plugin': ['example']})
-    requiredFormErrors = ['name', 'This field is required.']
+    POST = MultiValueDict({'name': [''], 'plugin': ['/plugins/example']})
+    requiredFormErrors = ['Name is required.']
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
 
@@ -137,8 +141,8 @@ class TestServiceCreateViewNatashaPOSTErrorInvalid(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['/'], 'plugin': ['example']})
-    requiredFormErrors = ['name', 'Enter a valid value.']
+    POST = MultiValueDict({'name': ['/'], 'plugin': ['/plugins/example']})
+    requiredFormErrors = ['Enter a valid name.']
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
 
@@ -146,8 +150,8 @@ class TestServiceCreateViewNatashaPOSTErrorInUse(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['example'], 'plugin': ['example']})
-    requiredFormErrors = "A service called 'example' already exists. Please choose another service name."
+    POST = MultiValueDict({'name': ['example'], 'plugin': ['/plugins/example']})
+    requiredFormErrors = "The name 'example' is already being used."
     requiredResponseClassName = 'HttpResponseBadRequest'
     requiredRedirect = ''
 
@@ -155,86 +159,59 @@ class TestServiceCreateViewNatashaPOST(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['example2'], 'plugin': ['example']})
-    requiredRedirect = '/project/warandpeace/services/example2/'
-    if ServiceView.isReloadingModWsgi():
-        requiredResponseClassName = 'HttpResponse'
-    else:
-        requiredResponseClassName = 'HttpResponseRedirect'
+    POST = MultiValueDict({'name': ['example2'], 'plugin': ['/plugins/example']})
+    requiredRedirect = '/projects/warandpeace/services/example2/'
+    requiredResponseClassName = 'HttpResponseRedirect'
     requiredFormErrors = False
 
     def tearDown(self):
-        super(TestServiceCreateViewNatashaPOST, self).tearDown()
         if 'example2' in self.registry.projects[self.projectName].services:
             service = self.registry.projects[self.projectName].services['example2']
             service.delete()
             service.purge()
+        super(TestServiceCreateViewNatashaPOST, self).tearDown()
 
 
 class TestServiceCreateViewNatashaPOSTSvn(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['svn'], 'plugin': ['svn']})
-    requiredRedirect = '/project/warandpeace/services/svn/'
-    if ServiceView.isReloadingModWsgi():
-        requiredResponseClassName = 'HttpResponse'
-    else:
-        requiredResponseClassName = 'HttpResponseRedirect'
+    POST = MultiValueDict({'name': ['svn'], 'plugin': ['/plugins/svn']})
+    requiredRedirect = '/projects/warandpeace/services/svn/'
+    requiredResponseClassName = 'HttpResponseRedirect'
     requiredFormErrors = False
-
-    def setUp(self):
-        super(TestServiceCreateViewNatashaPOSTSvn, self).setUp()
-        if 'svn' in self.registry.plugins:
-            self.enabledSvnPlugin = False
-        else: 
-            self.registry.plugins.create('svn')
-            self.enabledSvnPlugin = True
+    requiredPlugins = ['svn']
 
     def tearDown(self):
-        super(TestServiceCreateViewNatashaPOSTSvn, self).tearDown()
         if 'svn' in self.registry.projects[self.projectName].services:
             service = self.registry.projects[self.projectName].services['svn']
             service.delete()
             service.purge()
-        if self.enabledSvnPlugin:
-            del(self.registry.plugins['svn'])
+        super(TestServiceCreateViewNatashaPOSTSvn, self).tearDown()
 
 
 class TestServiceCreateViewNatashaPOSTTrac(ServiceTestCase):
 
     viewerName = 'natasha'
     viewClass = ServiceCreateView
-    POST = MultiValueDict({'name': ['trac'], 'plugin': ['trac']})
-    requiredRedirect = '/project/warandpeace/services/trac/edit/'
-    if ServiceView.isReloadingModWsgi():
-        requiredResponseClassName = 'HttpResponse'
-    else:
-        requiredResponseClassName = 'HttpResponseRedirect'
+    POST = MultiValueDict({'name': ['trac'], 'plugin': ['/plugins/trac']})
+    requiredRedirect = '/projects/warandpeace/services/trac/'
+    requiredResponseClassName = 'HttpResponseRedirect'
     requiredFormErrors = False
+    requiredPlugins = ['trac']
 
-    def setUp(self):
-        super(TestServiceCreateViewNatashaPOSTTrac, self).setUp()
-        if 'trac' in self.registry.plugins:
-            self.enabledTracPlugin = False
-        else: 
-            self.registry.plugins.create('trac')
-            self.enabledTracPlugin = True
+    def tearDown(self):
+        if 'trac' in self.registry.projects[self.projectName].services:
+            service = self.registry.projects[self.projectName].services['trac']
+            service.delete()
+            service.purge()
+        super(TestServiceCreateViewNatashaPOSTTrac, self).tearDown()
 
     def test_getResponse(self):
         super(TestServiceCreateViewNatashaPOSTTrac, self).test_getResponse()
         service = self.registry.projects[self.projectName].services['trac']
         tracProject = service.getExtnObject()
-        self.failIf(tracProject.isEnvironmentInitialised, repr(tracProject))
-
-    def tearDown(self):
-        super(TestServiceCreateViewNatashaPOSTTrac, self).tearDown()
-        if 'trac' in self.registry.projects[self.projectName].services:
-            service = self.registry.projects[self.projectName].services['trac']
-            service.delete()
-            service.purge()
-        if self.enabledTracPlugin:
-            del(self.registry.plugins['trac'])
+        self.failUnless(tracProject.isEnvironmentInitialised, repr(tracProject))
 
 
 class TestServiceReadView(ServiceTestCase):
@@ -267,263 +244,145 @@ class TestServiceUpdateViewNatasha(ServiceTestCase):
     requiredResponseClassName = 'HttpResponse'
 
 
-class TestServiceUpdateViewNatashaPOST(ServiceTestCase):
+class TestServiceUpdateViewNatashaPOST(TestServiceUpdateViewNatasha):
 
-    viewerName = 'natasha'
-    serviceName = 'example2'
-    viewClass = ServiceUpdateView
-    POST = MultiValueDict({'name': ['example3']})
-    requiredRedirect = '/project/warandpeace/services/example3/'
+    POST = MultiValueDict({'submit': 'submit changes'})
+    requiredRedirect = '/projects/warandpeace/services/example/'
     requiredResponseClassName = 'HttpResponseRedirect'
-    if ServiceView.isReloadingModWsgi():
-        requiredResponseClassName = 'HttpResponse'
-    else:
-        requiredResponseClassName = 'HttpResponseRedirect'
     requiredFormErrors = False
 
-    def setUp(self):
-        if 'example2' not in self.registry.projects[self.projectName].services:
-            plugin = self.registry.plugins['example']
-            self.domainObject = self.registry.projects[self.projectName].services.create('example2', plugin=plugin)
-        super(TestServiceUpdateViewNatashaPOST, self).setUp()
-
-    def tearDown(self):
-        super(TestServiceUpdateViewNatashaPOST, self).tearDown()
-        if 'example2' in self.registry.projects[self.projectName].services:
-            service = self.registry.projects[self.projectName].services['example2']
-            service.delete()
-            service.purge()
-        if 'example3' in self.registry.projects[self.projectName].services:
-            service = self.registry.projects[self.projectName].services['example3']
-            service.delete()
-            service.purge()
-
 
 class TestServiceUpdateViewNatashaPOSTTrac(TestServiceUpdateViewNatashaPOST):
     # Check update trac service, without setting repository.
 
     serviceName = 'trac'
-    requiredRedirect = ''
-    POST = MultiValueDict({'name': ['trac1']})
-    requiredRedirect = '/project/warandpeace/services/trac1/'
+    requiredRedirect = '/projects/warandpeace/services/trac/'
+    requiredPlugins = ['trac']
 
-    def setUp(self):
-        if 'svn' in self.registry.plugins:
-            svnPlugin = self.registry.plugins['svn']
-            self.enabledSvnPlugin = False
-        else: 
-            svnPlugin = self.registry.plugins.create('svn')
-            self.enabledSvnPlugin = True
-        if 'svn' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['svn']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('svn', plugin=svnPlugin)
-        if 'trac' in self.registry.plugins:
-            tracPlugin = self.registry.plugins['trac']
-            self.enabledTracPlugin = False
-        else: 
-            tracPlugin = self.registry.plugins.create('trac')
-            self.enabledTracPlugin = True
-        if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('trac', plugin=tracPlugin)
-        super(TestServiceUpdateViewNatashaPOSTTrac, self).setUp()
-    
     def initPost(self):
-        self.POST['name'] = 'trac1'
+        tracService = self.createTracService()
+        tracProject = self.getTracProject()
+        self.failUnless(tracProject.isEnvironmentInitialised)
+        self.failUnless(tracProject.isOperational())
+        configfileText = tracProject.asDictValues()['configfile']
+        self.POST = MultiValueDict({})
+        self.POST['configfile'] = configfileText
 
+    def tearDown(self):
+        self.destroyTracService()
+        super(TestServiceUpdateViewNatashaPOSTTrac, self).tearDown()
+
+    def createTracService(self):
+        tracPlugin = self.registry.plugins['trac']
+        project = self.registry.projects[self.projectName]
+        return project.services.create('trac', plugin=tracPlugin)
+
+    def getTracProject(self):
+        project = self.registry.projects[self.projectName]
+        if 'trac' not in project.services:
+            raise Exception, "No service named 'trac' in project services: %s" % project.services.keys()
+        service = project.services['trac']
+        return service.getExtnObject()
+
+    def destroyTracService(self):
+        project = self.registry.projects[self.projectName]
+        if 'trac' in project.services:
+            service = project.services['trac']
+            service.delete()
+            service.purge()
+    
     def test_getResponse(self):
         super(TestServiceUpdateViewNatashaPOSTTrac, self).test_getResponse()
-        service = self.registry.projects[self.projectName].services['trac1']
-        tracProject = service.getExtnObject()
-        self.failIf(tracProject.isEnvironmentInitialised, repr(tracProject))
-        
-    def tearDown(self):
-        super(TestServiceUpdateViewNatashaPOSTTrac, self).tearDown()
-        try:
-            if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-            if 'trac1' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac1']
-                service.delete()
-                service.purge()
-            if 'svn' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['svn']
-                service.delete()
-                service.purge()
-        finally:
-            try:
-                if self.enabledSvnPlugin:
-                    plugin = self.registry.plugins['svn']
-                    plugin.delete()
-                    plugin.purge()
-            finally:
-                if self.enabledTracPlugin:
-                    plugin = self.registry.plugins['trac']
-                    plugin.delete()
-                    plugin.purge()
+        tracProject = self.getTracProject()
+        self.failUnless(tracProject.isEnvironmentInitialised)
+        self.failUnless(tracProject.isOperational())
+        configfileText = tracProject.asDictValues()['configfile']
+        self.failUnless(configfileText)
+        repositories = tracProject.asDictValues()['repositories']
+        self.checkRepositories(repositories)
+
+    def checkRepositories(self, repositories):
+        self.failIf(repositories)
 
 
-class TestServiceUpdateViewNatashaPOSTTracSvn(TestServiceUpdateViewNatashaPOST):
+class TestServiceUpdateViewNatashaPOSTTracSvn(TestServiceUpdateViewNatashaPOSTTrac):
     # Check update trac service, with setting repository (svn).
 
-    serviceName = 'trac'
-    requiredRedirect = ''
-    POST = MultiValueDict({'name': ['trac']})
-    requiredRedirect = '/project/warandpeace/services/trac/'
+    requiredPlugins = ['trac', 'svn']
 
-    def setUp(self):
-        if 'svn' in self.registry.plugins:
-            svnPlugin = self.registry.plugins['svn']
-            self.enabledSvnPlugin = False
-        else: 
-            svnPlugin = self.registry.plugins.create('svn')
-            self.enabledSvnPlugin = True
-        if 'svn' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['svn']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('svn', plugin=svnPlugin)
-        if 'trac' in self.registry.plugins:
-            tracPlugin = self.registry.plugins['trac']
-            self.enabledTracPlugin = False
-        else: 
-            tracPlugin = self.registry.plugins.create('trac')
-            self.enabledTracPlugin = True
-        if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('trac', plugin=tracPlugin)
-        super(TestServiceUpdateViewNatashaPOSTTracSvn, self).setUp()
-    
     def initPost(self):
-        self.POST['svn'] = self.registry.projects[self.projectName].services['svn'].id
+        super(TestServiceUpdateViewNatashaPOSTTracSvn, self).initPost()
+        project = self.registry.projects[self.projectName]
+        svnPlugin = self.registry.plugins['svn']
+        svnService = project.services.create('svn', plugin=svnPlugin)
+        self.failUnless(svnService.getUri())
+        self.POST['repositories'] = [svnService.getUri()]
 
-    def test_getResponse(self):
-        super(TestServiceUpdateViewNatashaPOSTTracSvn, self).test_getResponse()
-        service = self.registry.projects[self.projectName].services['trac']
-        tracProject = service.getExtnObject()
-        self.failUnless(tracProject.isEnvironmentInitialised, repr(tracProject))
-        
     def tearDown(self):
+        self.destroySvnService()
         super(TestServiceUpdateViewNatashaPOSTTracSvn, self).tearDown()
-        try:
-            if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-            if 'svn' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['svn']
-                service.delete()
-                service.purge()
-        finally:
-            try:
-                if self.enabledSvnPlugin:
-                    plugin = self.registry.plugins['svn']
-                    plugin.delete()
-                    plugin.purge()
-            finally:
-                if self.enabledTracPlugin:
-                    plugin = self.registry.plugins['trac']
-                    plugin.delete()
-                    plugin.purge()
 
-
-class TestServiceUpdateViewNatashaPOSTTracHg(TestServiceUpdateViewNatashaPOST):
-    # Check update trac service, with setting repository (svn).
-
-    serviceName = 'trac'
-    requiredRedirect = ''
-    POST = MultiValueDict({'name': ['trac']})
-    requiredRedirect = '/project/warandpeace/services/trac/'
-
-    def setUp(self):
-        if 'mercurial' in self.registry.plugins:
-            hgPlugin = self.registry.plugins['mercurial']
-            self.enabledHgPlugin = False
-        else: 
-            hgPlugin = self.registry.plugins.create('mercurial')
-            self.enabledHgPlugin = True
-        if 'mercurial' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['mercurial']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('mercurial', plugin=hgPlugin)
-        if 'trac' in self.registry.plugins:
-            tracPlugin = self.registry.plugins['trac']
-            self.enabledTracPlugin = False
-        else: 
-            tracPlugin = self.registry.plugins.create('trac')
-            self.enabledTracPlugin = True
-        if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-        self.registry.projects[self.projectName].services.create('trac', plugin=tracPlugin)
-        super(TestServiceUpdateViewNatashaPOSTTracHg, self).setUp()
+    def destroySvnService(self):
+        project = self.registry.projects[self.projectName]
+        if 'svn' in project.services:
+            service = project.services['svn']
+            service.delete()
+            service.purge()
     
-    def initPost(self):
-        self.POST['svn'] = self.registry.projects[self.projectName].services['mercurial'].id
+    def checkRepositories(self, repositories):
+        self.failUnless(repositories)
 
-    def test_getResponse(self):
-        super(TestServiceUpdateViewNatashaPOSTTracHg, self).test_getResponse()
-        service = self.registry.projects[self.projectName].services['trac']
-        tracProject = service.getExtnObject()
-        self.failUnless(tracProject.isEnvironmentInitialised, repr(tracProject))
-        
+
+class TestServiceUpdateViewNatashaPOSTTracHg(TestServiceUpdateViewNatashaPOSTTrac):
+    # Check update trac service, with setting repository (hg).
+
+    requiredPlugins = ['trac', 'mercurial']
+
+    def initPost(self):
+        super(TestServiceUpdateViewNatashaPOSTTracHg, self).initPost()
+        project = self.registry.projects[self.projectName]
+        hgPlugin = self.registry.plugins['mercurial']
+        hgService = project.services.create('hg', plugin=hgPlugin)
+        self.failUnless(hgService.getUri())
+        self.POST['repositories'] = [hgService.getUri()]
 
     def tearDown(self):
+        self.destroyHgService()
         super(TestServiceUpdateViewNatashaPOSTTracHg, self).tearDown()
-        try:
-            if 'trac' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['trac']
-                service.delete()
-                service.purge()
-            if 'mercurial' in self.registry.projects[self.projectName].services:
-                service = self.registry.projects[self.projectName].services['mercurial']
-                service.delete()
-                service.purge()
-        finally:
-            try:
-                if self.enabledHgPlugin:
-                    plugin = self.registry.plugins['mercurial']
-                    plugin.delete()
-                    plugin.purge()
-            finally:
-                if self.enabledTracPlugin:
-                    plugin = self.registry.plugins['trac']
-                    plugin.delete()
-                    plugin.purge()
 
-
-class TestServiceUpdateViewNatashaPOSTErrorInUse(TestServiceUpdateViewNatashaPOST):
-
-    POST = MultiValueDict({'name': ['example']})
-    requiredRedirect = ''
-    requiredResponseClassName = 'HttpResponseBadRequest'
-    requiredFormErrors = "A service called 'example' already exists. Please choose another service name."
+    def destroyHgService(self):
+        project = self.registry.projects[self.projectName]
+        if 'hg' in project.services:
+            service = project.services['hg']
+            service.delete()
+            service.purge()
     
+    def checkRepositories(self, repositories):
+        self.failUnless(repositories)
 
-class TestServiceUpdateViewNatashaPOSTErrorRequired(TestServiceUpdateViewNatashaPOST):
 
-    POST = MultiValueDict({'name': ['']})
-    requiredRedirect = ''
-    requiredResponseClassName = 'HttpResponseBadRequest'
-    requiredFormErrors = ['name', 'This field is required.']
-    
-
-class TestServiceUpdateViewNatashaPOSTErrorInvalid(TestServiceUpdateViewNatashaPOST):
-
-    POST = MultiValueDict({'name': [' ']})
-    requiredRedirect = ''
-    requiredResponseClassName = 'HttpResponseBadRequest'
-    requiredFormErrors = ['name', 'Enter a valid value.']
+#class TestServiceUpdateViewNatashaPOSTErrorInUse(TestServiceUpdateViewNatashaPOST):
+#
+#    POST = MultiValueDict({'name': ['example']})
+#    requiredRedirect = ''
+#    requiredResponseClassName = 'HttpResponseBadRequest'
+#    requiredFormErrors = "The name 'example' is already being used."
+#    
+#
+#class TestServiceUpdateViewNatashaPOSTErrorRequired(TestServiceUpdateViewNatashaPOST):
+#
+#    POST = MultiValueDict({'name': ['']})
+#    requiredRedirect = ''
+#    requiredResponseClassName = 'HttpResponseBadRequest'
+#    requiredFormErrors = ['Name is required.']
+#    
+#
+#class TestServiceUpdateViewNatashaPOSTErrorInvalid(TestServiceUpdateViewNatashaPOST):
+#
+#    POST = MultiValueDict({'name': [' ']})
+#    requiredRedirect = ''
+#    requiredResponseClassName = 'HttpResponseBadRequest'
+#    requiredFormErrors = ['Enter a valid name.']
     
 
 class TestServiceDeleteView(ServiceTestCase):
@@ -549,11 +408,12 @@ class TestServiceDeleteViewNatashaPOST(ServiceTestCase):
     serviceName = 'example2'
     viewClass = ServiceDeleteView
     POST = MultiValueDict({'Submit': ['submit']})
-    requiredRedirect = '/project/warandpeace/services/'
-    if ServiceView.isReloadingModWsgi():
-        requiredResponseClassName = 'HttpResponse'
-    else:
-        requiredResponseClassName = 'HttpResponseRedirect'
+    requiredRedirect = '/projects/warandpeace/'
+    #if ServiceCreateView.isReloadingModWsgi():
+    #    requiredResponseClassName = 'HttpResponse'
+    #else:
+    #    requiredResponseClassName = 'HttpResponseRedirect'
+    requiredResponseClassName = 'HttpResponseRedirect'
     requiredFormErrors = False
 
     def setUp(self):
@@ -569,3 +429,4 @@ class TestServiceDeleteViewNatashaPOST(ServiceTestCase):
             service.delete()
             service.purge()
             self.fail("Project still has a service called 'example2'.")
+            
