@@ -13,12 +13,8 @@ class TestWelcomeView(ViewTestCase):
 
     viewClass = WelcomeView
     requiredResponseContent = [
-        "Welcome to",
-        "What is KForge",
-        "Site summary",
-        "Quick links",
-        "Using KForge",
-        "KForge Club",
-
+        "Follow projects",
+        "Sign up",
+        "Make changes",
     ]
 
