@@ -36,7 +36,7 @@ class UmaskSetter(object):
         self._oldUmask = None
 
 
-umaskSetter = UmaskSetter(2)
+umaskSetter = UmaskSetter(application.dictionary.getUmask())
 if not hasattr(dispatcher, 'connect'):  # Django 1.0.
     signals.request_started.connect(umaskSetter.beginRequest)
     signals.request_finished.connect(umaskSetter.endRequest)
