@@ -18,7 +18,7 @@ class TestFeedEntry(TestCase):
 
     def test_update(self):
         self.feed.readSources([
-            'http://knowledgeforge.net/kforge/trac/timeline?ticket=on&changeset=on&milestone=on&wiki=on&max=50&daysback=90&format=rss',
-            'http://knowledgeforge.net/domainmodel/trac/timeline?ticket=on&changeset=on&milestone=on&wiki=on&max=50&daysback=90&format=rss',
+            'http://kforge.appropriatesoftware.net/kforge/trac/timeline?ticket=on&changeset=on&milestone=on&wiki=on&max=50&daysback=90&format=rss',
+            'http://kforge.appropriatesoftware.net/domainmodel/trac/timeline?ticket=on&changeset=on&milestone=on&wiki=on&max=50&daysback=90&format=rss',
         ])
 
