@@ -1,5 +1,6 @@
 from dm.dom.plugin import Plugin
-from dm.dom.stateful import HasManyPages 
+#from dm.dom.stateful import HasManyPages 
+from dm.dom.stateful import HasMany
 from dm.ioc import RequiredFeature
 from kforge.exceptions import MissingPluginSystem
 
@@ -10,7 +11,13 @@ def getProjects():
 class Plugin(Plugin):
     "Registered Plugin."
 
-    services = HasManyPages('Service', 'name', 'project', pageKeys=getProjects)
+    #services = HasManyPages('Service', 'name', 'project', pageKeys=getProjects)
+    services = HasMany('Service', 'id')
+
+    def delete(self, *args, **kwds):
+        if len(self.services):
+            raise Exception, "Unable to delete plugin whilst there are active services based on this plugin."
+        super(Plugin, self).delete()
 
     def getMaxServicesPerProject(self):
         "Returns the maximum service instances for any project."
@@ -31,6 +38,10 @@ class Plugin(Plugin):
         self.assertHasSystem()
         return self.getSystem().buildCgiFile()
 
+    def buildLocksDir(self):
+        self.assertHasSystem()
+        return self.getSystem().buildLocksDir()
+
     def assertHasSystem(self):
         if not self.hasSystem():
             msg = "Plugin object '%s' has no system object." % self.name
@@ -40,3 +51,6 @@ class Plugin(Plugin):
     def hasSystem(self):
         return bool(self.getSystem())
 
+    def isReadyToReload(self, service):
+        system = self.getSystem()
+        return system and system.isReadyToReload(service)
