@@ -1,3 +1,4 @@
+import kforge.dom.persontest
 import kforge.dom.projecttest
 import kforge.dom.membertest
 import kforge.dom.servicetest
@@ -10,6 +11,7 @@ def suite():
     suites = [
         unittest.makeSuite(TestDomainRegistry),
         unittest.makeSuite(TestPluginNotification),
+        kforge.dom.persontest.suite(),
         kforge.dom.projecttest.suite(),
         kforge.dom.membertest.suite(),
         kforge.dom.servicetest.suite(),
@@ -32,9 +34,9 @@ class TestDomainRegistry(TestCase):
             project.purge()
         except:
             pass
-        persons = self.registry.persons
+        people = self.registry.people
         try:
-            person = persons['TestDomainRegistry']
+            person = people['TestDomainRegistry']
             person.delete()
             person.purge()
         except:
@@ -63,14 +65,14 @@ class TestDomainRegistry(TestCase):
             project.purge()
 
     def test_person(self):
-        persons = self.registry.persons
-        person = persons.create('TestDomainRegistry')
+        people = self.registry.people
+        person = people.create('TestDomainRegistry')
         try:
             self.failUnless(person, "No person created.")
-            self.failUnless(persons['TestDomainRegistry'], "No person found.")
-            self.failIf(persons.has_key('TestDomainRegistry2'), "Person found.")
+            self.failUnless(people['TestDomainRegistry'], "No person found.")
+            self.failIf(people.has_key('TestDomainRegistry2'), "Person found.")
             try:
-                persons['TestDomainRegistry2']
+                people['TestDomainRegistry2']
             except KforgeRegistryKeyError:
                 pass
             else:
@@ -102,17 +104,11 @@ class TestDomainRegistry(TestCase):
         finally:
             plugin.delete()
 
-    def test_service(self):
-        registry = self.registry.services
-
-    def test_member(self):
-        registry = self.registry.members
-    
     def testMember(self):
         self.project = self.registry.projects.create('TestDomainRegistry')
         try:
-            self.person  = self.registry.persons.create('TestDomainRegistry')
-            self.person2 = self.registry.persons.create('TestDomainRegistry2')
+            self.person  = self.registry.people.create('TestDomainRegistry')
+            self.person2 = self.registry.people.create('TestDomainRegistry2')
         except:
             self.project.delete()
             self.project.purge()
@@ -145,11 +141,11 @@ class TestPluginNotification(TestCase):
     def setUp(self):
         super(TestPluginNotification, self).setUp()
         self.fixtureName = 'TestPluginNotification'
-        self.persons = self.registry.persons
+        self.people = self.registry.people
 
     def tearDown(self):
         try:
-            person = self.persons[self.fixtureName]
+            person = self.people[self.fixtureName]
             person.delete()
             person.purge()
         except:
@@ -161,7 +157,7 @@ class TestPluginNotification(TestCase):
         self.examplePlugin = self.registry.plugins['example']
 
         before = self.examplePlugin.getSystem().counts['onPersonCreate']
-        self.person = self.persons.create(self.fixtureName)
+        self.person = self.people.create(self.fixtureName)
         after = self.examplePlugin.getSystem().counts['onPersonCreate']
         self.failUnless(before + 1 == after)
         
