@@ -1,9 +1,8 @@
-import os.path
-import dm.filesystem
 from kforge.dictionarywords import PROJECTS_PATH
-from kforge.dictionarywords import DAV_LOCK_PATH
+from dm.filesystem import FileSystem as BaseFileSystem
+import os.path
 
-class FileSystemPathBuilder(dm.filesystem.FileSystemPathBuilder):
+class FileSystem(BaseFileSystem):
     """
     Directory layout:
     =================
@@ -25,7 +24,7 @@ class FileSystemPathBuilder(dm.filesystem.FileSystemPathBuilder):
     """
 
     def __init__(self):
-        super(FileSystemPathBuilder, self).__init__()
+        super(FileSystem, self).__init__()
         path = self.dictionary[PROJECTS_PATH]
         self.projectDataPath = os.path.normpath(path)
 
@@ -41,17 +40,6 @@ class FileSystemPathBuilder(dm.filesystem.FileSystemPathBuilder):
         """
         return os.path.join(self.projectDataPath, '__locks__')
  
-    def getDavLockPath(self):
-        """
-        Returns filesystem path for DAV lock.
-        """
-        lockPath = self.dictionary[DAV_LOCK_PATH]
-        if not lockPath:
-            locksPath = self.getLocksPath()
-            self.assertFolder(locksPath, 'locks')
-            lockPath = os.path.join(locksPath, 'DAV.lock')
-        return lockPath
- 
     def getProjectPath(self, project):
         """
         Returns filesystem path for a project.
