@@ -1,18 +1,18 @@
-import kforge.filesystem
+from kforge.filesystem import FileSystem
 import unittest
 from kforge.testunit import TestCase
 
 def suite():
     suites = [
-        unittest.makeSuite(FileSystemPathBuilderTest),
+        unittest.makeSuite(FileSystemTest),
     ]
     return unittest.TestSuite(suites)
 
-class FileSystemPathBuilderTest(TestCase):
+class FileSystemTest(TestCase):
     
     def setUp(self):
-        super(FileSystemPathBuilderTest, self).setUp()
-        self.builder = kforge.filesystem.FileSystemPathBuilder()
+        super(FileSystemTest, self).setUp()
+        self.filesystem = FileSystem()
     
     def getAllServices(self):
         servicesList = []
@@ -23,17 +23,17 @@ class FileSystemPathBuilderTest(TestCase):
         
     def testGetProjectPath(self):
         for project in self.registry.projects:
-            path = self.builder.getProjectPath(project)
+            path = self.filesystem.getProjectPath(project)
             self.failUnless(project.name in path)
     
     def testGetProjectPluginPath(self):
         for service in self.getAllServices():
             project = service.project
             plugin = service.plugin
-            path = self.builder.getProjectPluginPath(project, plugin)
+            path = self.filesystem.getProjectPluginPath(project, plugin)
             self.failUnless(project.name in path)
     
     def testGetServicePath(self):
         for service in self.getAllServices():
-            self.builder.getServicePath(service)
+            self.filesystem.getServicePath(service)
  
