@@ -59,7 +59,7 @@ class MockPythonRequest(HttpRequest):
     def __init__(self, user='', method='GET', project='annakarenina', 
             service='example'):
         self.uri = kforge.url.UrlScheme().url_for(
-            'project.service', project=project, service=service)
+            'projects.service', project=project, service=service)
         self.method = method
         self.headers_in = MockTableRecord()
         self.headers_out = MockTableRecord()
@@ -85,6 +85,7 @@ class MockPythonRequest(HttpRequest):
     def write(self, bodyContent):
         self.bodyContent += bodyContent
 
+
 class BasicClientRequest(MockPythonRequest):
 
     passwords = {
@@ -98,7 +99,7 @@ class BasicClientRequest(MockPythonRequest):
         self.subprocess_env = {'HTTP_USER_AGENT': 'SVN'}
 
     def setUser(self, user):
-        self.user = user  # Just like in the movies.
+        self.user = user
 
     def get_basic_auth_pw(self):
         return self.passwords[self.user]
@@ -126,7 +127,7 @@ class CookieClientWithCookie(CookieClientAnonymous):
     def addSessionCookie(self, user):
         import dm.view.base
         view = dm.view.base.ControlledAccessView(None)
-        person = view.registry.persons[user]
+        person = view.registry.people[user]
         session = person.sessions.create()
         cookieString = view.makeCookieStringFromSessionKey(session.key)
         cookieName = view.dictionary[AUTH_COOKIE_NAME]
@@ -142,8 +143,8 @@ class HandlerTestCase(TestCase):
     requestMethod = 'GET'
     requestProject = 'annakarenina'
     requestService = 'example'
-    handlerClass = None
     requestClass = None
+    handlerClass = None
 
     requireCookieClient = False
     requireHandlerSession = None
