@@ -14,15 +14,34 @@ subdirectories.
 
 2. KForge config file. You do not need to add anything to the KForge config
    file.
-"""
-import os
-import shutil
 
+
+NB: If you see "Invalid method in request USERINFO" in Apache error log, this
+seems to be a result of using a particular client. It isn't a KForge error.
+
+"""
 import kforge.plugin.base
 from dm.strategy import FindProtectionObject
 from kforge.dictionarywords import *
+from kforge.plugin.base import dictionary, setWord
+from kforge.ioc import RequiredFeature
+import os
+import shutil
+
+filesystem = RequiredFeature('FileSystem')
+DAV_LOCK_PATH = DavLockSetting('dav.lock_path')
+DAV_WSGI_SCRIPT_PATH = WsgiScriptSetting('dav.wsgi_script')
+DAV_WSGI_PROCESS_GROUP = DictionaryDef('dav.wsgi_process_group')
+defaultLocksPath = os.path.join(dictionary[FILESYSTEM_PATH], 'var', 'locks', 'DAV.lock')
+defaultWsgiScriptPath = os.path.join(dictionary[FILESYSTEM_PATH], 'wsgi', 'dav.wsgi')
+defaultWsgiProcessGroup = dictionary[WSGI_PROCESS_GROUP]
+setWord(DAV_LOCK_PATH, defaultLocksPath)
+setWord(DAV_WSGI_SCRIPT_PATH, defaultWsgiScriptPath)
+setWord(DAV_WSGI_PROCESS_GROUP, defaultWsgiProcessGroup)
 
 class Plugin(kforge.plugin.base.SingleServicePlugin):
+
+    settings = [DAV_LOCK_PATH, DAV_WSGI_SCRIPT_PATH, DAV_WSGI_PROCESS_GROUP]
     
     def __init__(self, domainObject):
         super(Plugin, self).__init__(domainObject)
@@ -30,34 +49,48 @@ class Plugin(kforge.plugin.base.SingleServicePlugin):
     def onServiceCreate(self, service):
         if self.isOurs(service):
             self.paths.assertProjectFolder(service.project)
+            self.buildAndReloadApacheConfig()
 
-    def getApacheConfig(self, service):
+    def getApacheConfigCommon(self):
+        configVars = {'lockFilePath': self.dictionary[DAV_LOCK_PATH]}
+        configTmpl = """
+DavLockDB %(lockFilePath)s
+"""
+        return configTmpl % configVars
+
+    def getApacheConfig(self, service, configVars):
         projectPath = self.paths.getProjectPath(service.project)
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            config = """
-WSGIScriptAlias %(urlPath)s """ + self.dictionary[DAV_WSGI_SCRIPT_PATH] + """
+        configVars['wsgiScriptPath'] = self.dictionary[DAV_WSGI_SCRIPT_PATH]
+        configVars['wsgiProcessGroup'] = self.dictionary[DAV_WSGI_PROCESS_GROUP]
+        configVars['projectPath'] = self.paths.getProjectPath(service.project)
+        apacheConfigTmpl = """
+<IfModule mod_wsgi.c>
+WSGIScriptAlias %(urlPath)s %(wsgiScriptPath)s
 WSGIApplicationGroup %%{GLOBAL}
-WSGIProcessGroup """ + self.dictionary[DAV_WSGI_PROCESS_GROUP] + """
+WSGIProcessGroup %(wsgiProcessGroup)s
 <Location %(urlPath)s>
-SetEnv KFORGE_DAV_FILESYSTEM_PATH """ + projectPath + """
+SetEnv KFORGE_DAV_FILESYSTEM_PATH %(projectPath)s
 SetEnv KFORGE_DAV_MOUNT_PATH %(urlPath)s
 WSGIPassAuthorization On
 </Location>
-"""
-        elif self.dictionary[APACHE_PYTHON_MODULE] == 'mod_python':
-            config = 'Alias %(urlPath)s ' + projectPath
-            # 'ForceType text/plain' so DAV interprets all file types as text.
-            config += """
-    <Location %(urlPath)s>
-    DAV On
-    Options +Indexes
-# Remove use DirectoryIndex
-    DirectoryIndex none.none.none
-    ForceType text/plain
+</IfModule>
 
-    %(accessControl)s
-    </Location>"""
-        return config
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
+Alias %(urlPath)s %(projectPath)s
+<Location %(urlPath)s>
+DAV On
+Options +Indexes
+# Remove use DirectoryIndex
+DirectoryIndex none.none.none
+# 'ForceType text/plain' so DAV interprets all file types as text.
+ForceType text/plain
+%(modPythonAccessControl)s
+</Location>
+</IfModule>
+</IfModule>
+"""
+        return apacheConfigTmpl % configVars
     
     def onCreate(self):
         # Todo: Pull various bits of access controller setup together under the name 'setupAccessControl'.
@@ -77,10 +110,23 @@ WSGIPassAuthorization On
             grant.delete()
 
     def buildWsgiFile(self):
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            wsgiScriptPath = self.dictionary[DAV_WSGI_SCRIPT_PATH]
-            wsgiScriptContent = self.createWsgiScriptContent()
-            self.writeFile(wsgiScriptPath, wsgiScriptContent, 'DAV WSGI script')
+        path = self.dictionary[DAV_WSGI_SCRIPT_PATH]
+        content = self.createWsgiScriptContent()
+        purpose = 'DAV WSGI script'
+        self.filesystem.writeWsgiFile(path, content, purpose)
+
+    def buildLocksDir(self):
+        # Create dirs if they don't exist.
+        lockDirPath = os.path.dirname(self.dictionary[DAV_LOCK_PATH])
+        if not os.path.exists(lockDirPath):
+            # Parents need to have permission 750.
+            umask = os.umask(0o027)
+            if not os.path.dirname(lockDirPath):
+                os.makedirs(os.path.dirname(lockDirPath))
+            # Locks directory needs to have permission 770.
+            os.umask(0o007)
+            os.makedirs(lockDirPath)
+            os.umask(umask)
 
     def createWsgiScriptBody(self, pythonPathActivation):
         wsgiScriptBody = """
@@ -88,42 +134,30 @@ WSGIPassAuthorization On
 
 import os
 import sys
-
 """
         wsgiScriptBody += pythonPathActivation
         wsgiScriptBody += """
-
 from wsgidav.wsgidav_app import DEFAULT_CONFIG, WsgiDAVApp
 
 def application(environ, start_response):
     os.environ['KFORGE_SETTINGS'] = '""" + self.dictionary[SYSTEM_CONFIG_PATH] + "'"
-
         wsgiScriptBody += """
     from kforge.handlers.modwsgi import WsgiAccessControlHandler
+    davFsPath = environ.pop('KFORGE_DAV_FILESYSTEM_PATH')
     davConfig = DEFAULT_CONFIG.copy()
     davConfig.update({
-        'provider_mapping': {
-            '/': environ.pop('KFORGE_DAV_FILESYSTEM_PATH'),
-        },
+        'provider_mapping': {'/': davFsPath},
         'mount_path': environ.pop('KFORGE_DAV_MOUNT_PATH'),
+        'verbose': 0,
     })
-    accessapplication = WsgiAccessControlHandler(WsgiDAVApp(davConfig))
+    davapplication = WsgiDAVApp(davConfig)
+    accessapplication = WsgiAccessControlHandler(davapplication)
     return accessapplication(environ, start_response)
 """    
         return wsgiScriptBody
 
     def getUserHelp(self, service, serviceLocation):
         values = {'httpUrl' : serviceLocation}
-        if 'ssh' in self.registry.plugins:
-            import getpass
-            sshUrl = 'ssh://%(sshUser)s@%(sshHost)s/%(project)s/%(service)s'
-            sshUrl %= {
-                'sshUser': self.dictionary[SSH_USER_NAME] or getpass.getuser(),
-                'sshHost': self.dictionary[SSH_HOST_NAME],
-                'project': service.project.name,
-                'service': service.name
-            }
-            values['sshUrl'] = sshUrl
         helpMessage = '''
 <p>This service provides a WebDAV filesystem. To find out more about <a href="http://www.webdav.org/">WebDAV</a> see the <a href="http://www.webdav.org/other/faq.html">WebDAV FAQ</a>.</p>
 <p>You can access the material in the filesystem in the following ways (provided you have sufficient permission):</p>
