@@ -9,13 +9,16 @@ Creating services with this plugin requires:
 
 Providing access to this plugin's services requires:
 
-  * Apache. The mod_python, and mod_dav Apache modules are used to provide access to Git through Apache. For example, on Debian the libapache2-mod-python and libapache2-svn packages must be installed and enabled.
+  * Apache. The mod_python or mod_wsgi Apache modules are used to provide access to Git through Apache. For example, on Debian the libapache2-mod-python or libapache2-mod-wsgi must be installed and enabled.
 
 Using Git repositories with Trac projects (optional) requires:
 
   * Trac's Git plugin. For example, on Debian the trac-git package can be installed.
 
-You do not need to add anything to the KForge config file.
+There are various configuration options (see KForge configuration file). For
+example, if the directory for "static" git files is not located at
+'/usr/share/gitweb/static' then you may need to change this path to be
+'/usr/share/gitweb' instead. This is one difference between Ubuntu and Debian.
 
 You can enable, disable, and show status in the usual way.
 
@@ -25,41 +28,38 @@ You can enable, disable, and show status in the usual way.
 """
 # Todo: Fix the helpMessage so it's https when necessary.
 from kforge.dictionarywords import *
-from kforge.dictionary import *  # Is this necessary?
 import kforge.plugin.base
+from kforge.plugin.ssh import SSH_USER_NAME
+from kforge.plugin.ssh import SSH_HOST_NAME
 import os
 import commands
 
+# Dictionary words.
+from kforge.plugin.base import dictionary, setWord
+GIT_ADMIN_SCRIPT = ExecutableFileSetting('git.admin_script', umask=0o22)
+GIT_HTTP_BACKEND_SCRIPT = ExecutableFileSetting('git.http_backend_script', umask=0o22)
+GIT_GITWEB_SCRIPT = ExecutableFileSetting('git.gitweb_script', umask=0o22)
+GIT_GITWEB_STATIC = DirSetting('git.gitweb_static')
+GIT_WSGI_SCRIPT_PATH = WsgiScriptSetting('git.wsgi_file')
+GIT_WSGI_PROCESS_GROUP = 'git.wsgi_process_group'
+defaultAdminScript = '/usr/bin/git'
+defaultHttpBackendScript = '/usr/lib/git-core/git-http-backend'
+defaultGitwebScript = '/usr/lib/cgi-bin/gitweb.cgi'
+defaultGitwebStatic = '/usr/share/gitweb/static'
+defaultWsgiScript = os.path.join(dictionary[FILESYSTEM_PATH], 'wsgi', 'git.wsgi')
+defaultWsgiProcessGroup = dictionary[WSGI_PROCESS_GROUP]
+setWord(GIT_ADMIN_SCRIPT, defaultAdminScript)
+setWord(GIT_HTTP_BACKEND_SCRIPT, defaultHttpBackendScript)
+setWord(GIT_GITWEB_SCRIPT, defaultGitwebScript)
+setWord(GIT_GITWEB_STATIC, defaultGitwebStatic)
+setWord(GIT_WSGI_SCRIPT_PATH, defaultWsgiScript)
+setWord(GIT_WSGI_PROCESS_GROUP, defaultWsgiProcessGroup)
+
+
 class Plugin(kforge.plugin.base.ServicePlugin):
+
+    settings = [GIT_ADMIN_SCRIPT, GIT_HTTP_BACKEND_SCRIPT, GIT_GITWEB_SCRIPT, GIT_GITWEB_STATIC, GIT_WSGI_SCRIPT_PATH, GIT_WSGI_PROCESS_GROUP]
     
-    def checkDependencies(self):
-        errors = []
-        path = self.dictionary[GIT_ADMIN_SCRIPT]
-        cmd = 'which %s' % path
-        (status, output) = commands.getstatusoutput(cmd)
-        if status:
-            errors.append("Couldn't find Git admin script '%s' on path." % path)
-        if not os.path.isfile(self.dictionary[GIT_GITWEB_SCRIPT]):
-            errors.append("Couldn't find Git web script: %s" % self.dictionary[GIT_GITWEB_SCRIPT])
-        if not os.path.isfile(self.dictionary[GIT_HTTP_BACKEND_SCRIPT]):
-            errors.append("Couldn't find Git HTTP backend script: %s" % self.dictionary[GIT_HTTP_BACKEND_SCRIPT])
-        return errors
-
-    def showDepends(self):
-        path = self.dictionary[GIT_ADMIN_SCRIPT]
-        cmd = 'which %s' % path
-        (status, output) = commands.getstatusoutput(cmd)
-        return [
-            "Admin script: %s" % (status and "Not found!" or ("Found OK. %s" % output)),
-            "HTTP backend: %s" % (os.path.isfile(self.dictionary[GIT_HTTP_BACKEND_SCRIPT]) and "Found OK. %s" % self.dictionary[GIT_HTTP_BACKEND_SCRIPT] or "Not Found"),
-            "Gitweb: %s" % (os.path.isfile(self.dictionary[GIT_GITWEB_SCRIPT]) and "Found OK. %s" % self.dictionary[GIT_GITWEB_SCRIPT] or "Not Found"),
-            "Apache cgi module: %s" % "Please check this module is enabled.",
-            "Apache alias module: %s" % "Please check this module is enabled.",
-            "Apache env module: %s" % "Please check this module is enabled.",
-            "Trac Git: %s" % "Please check this package is installed (optional).",
-        ]
-    showDepends = classmethod(showDepends)
-
     def onServiceCreate(self, service):
         if self.isOurs(service):
             servicePath = self.paths.getServicePath(service)
@@ -70,6 +70,7 @@ class Plugin(kforge.plugin.base.ServicePlugin):
             self.paths.assertServiceFolder(service)
             self.createRepository(servicePath, self.dictionary[GIT_ADMIN_SCRIPT])
             self.createGitwebConfig(servicePath, service.project.name, service.name)
+            self.buildAndReloadApacheConfig()
 
     def createRepository(self, path, gitCmd='git'):
         if not os.path.exists(path):
@@ -132,46 +133,49 @@ $favicon = "/gitweb_static/git-favicon.png";
 Alias /gitweb_static %(staticPath)s
 """ % kwds
 
-    def getApacheConfig(self, service):
+    def getApacheConfig(self, service, configVars):
         # Todo: Change all service URLs to have trailing slash (so config knows where they end
         # and they aren't at risk of shorter patterns matching longer names - current order is critical).
         # Todo: Support static file configuration, see "Accelerated static Apache 2.x" on this
         # page: http://www.kernel.org/pub/software/scm/git/docs/git-http-backend.html
         # - will need dictionary word, default value in dictionary, 
-        servicePath = self.paths.getServicePath(service)
-        backendPath = self.dictionary[GIT_HTTP_BACKEND_SCRIPT]
-        gitwebPath = self.dictionary[GIT_GITWEB_SCRIPT]
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            wsgiScriptPath = self.dictionary[GIT_WSGI_SCRIPT_PATH]
-            config = """
-WSGIScriptAlias %(urlPath)s """ + wsgiScriptPath + """
+        configVars['servicePath'] = self.paths.getServicePath(service)
+        configVars['backendPath'] = self.dictionary[GIT_HTTP_BACKEND_SCRIPT]
+        configVars['gitwebPath'] = self.dictionary[GIT_GITWEB_SCRIPT]
+        configVars['wsgiScriptPath'] = self.dictionary[GIT_WSGI_SCRIPT_PATH]
+        configVars['wsgiProcessGroup'] = self.dictionary[GIT_WSGI_PROCESS_GROUP]
+        apacheConfigTmpl = """
+<IfModule mod_wsgi.c>
+WSGIScriptAlias %(urlPath)s %(wsgiScriptPath)s
 WSGIApplicationGroup %%{GLOBAL}
-WSGIProcessGroup """ + self.dictionary[GIT_WSGI_PROCESS_GROUP] + """
+WSGIProcessGroup %(wsgiProcessGroup)s
 <Location %(urlPath)s>
-SetEnv GIT_PROJECT_ROOT """+servicePath+"""
+SetEnv GIT_PROJECT_ROOT %(servicePath)s
 SetEnv GIT_HTTP_EXPORT_ALL
-SetEnv GITWEB_CONFIG """+servicePath+"""/gitweb.conf
+SetEnv GITWEB_CONFIG %(servicePath)s/gitweb.conf
 WSGIPassAuthorization On
 </Location>
-"""
-        elif self.dictionary[APACHE_PYTHON_MODULE] == 'mod_python':
-            config = """
+</IfModule>
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
 <Location %(urlPath)s>
-SetEnv GIT_PROJECT_ROOT """+servicePath+"""
+SetEnv GIT_PROJECT_ROOT %(servicePath)s
 SetEnv GIT_HTTP_EXPORT_ALL
-SetEnv GITWEB_CONFIG """+servicePath+"""/gitweb.conf
-%(accessControl)s
+SetEnv GITWEB_CONFIG %(servicePath)s/gitweb.conf
+%(modPythonAccessControl)s
 </Location>
-ScriptAliasMatch "(?x)^%(urlPath)s(.*/(HEAD|info/refs|objects/(info/[^/]+|[0-9a-f]{2}/[0-9a-f]{38}|pack/pack-[0-9a-f]{40}\.(pack|idx))|git-(upload|receive)-pack))$" """+backendPath+"""/$1
-ScriptAlias %(urlPath)s """+gitwebPath+"""
+ScriptAliasMatch "(?x)^%(urlPath)s(.*/(HEAD|info/refs|objects/(info/[^/]+|[0-9a-f]{2}/[0-9a-f]{38}|pack/pack-[0-9a-f]{40}\.(pack|idx))|git-(upload|receive)-pack))$" %(backendPath)s/$1
+ScriptAlias %(urlPath)s %(gitwebPath)s
+</IfModule>
+</IfModule>
 """
-        return config
+        return apacheConfigTmpl % configVars
 
     def buildWsgiFile(self):
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            wsgiScriptPath = self.dictionary[GIT_WSGI_SCRIPT_PATH]
-            wsgiScriptContent = self.createWsgiScriptContent()
-            self.writeFile(wsgiScriptPath, wsgiScriptContent, 'Git WSGI script')
+        path = self.dictionary[GIT_WSGI_SCRIPT_PATH]
+        content = self.createWsgiScriptContent()
+        purpose = 'Git WSGI script'
+        self.filesystem.writeWsgiFile(path, content, purpose)
 
     def createWsgiScriptBody(self, pythonPathActivation):
         gitwebScriptPath = self.dictionary[GIT_GITWEB_SCRIPT]
@@ -236,32 +240,6 @@ def application(environ, start_response):
 """    
         return wsgiScriptBody
 
-
-
-#    def getApacheConfigCommon(self):
-#        path = self.paths.getDavLockPath()
-#        return """
-#        # Follows
-#        # <http://www.kernel.org/pub/software/scm/git/docs/howto/setup-git-server-over-http.txt>
-#        DAVLockDB "%(davLockPath)s"
-#""" % {'davLockPath': path}
-#
-#    def getApacheConfig(self, service):
-#        fsPath = self.paths.getServicePath(service)
-#        return """
-#        # Follows
-#        # <http://www.kernel.org/pub/software/scm/git/docs/howto/setup-git-server-over-http.txt>
-#        <Location %(urlPath)s>
-#            DAV On
-##            Options +Indexes +FollowSymLinks
-##            Allow from all
-##            Order allow,deny
-#            %(accessControl)s
-#        </Location>
-#        Alias %(urlPath)s """ + fsPath + """
-#"""
-
-
     def getUserHelp(self, service, serviceLocation):
         values = {'httpUrl' : serviceLocation}
         if 'ssh' in self.registry.plugins:
@@ -335,3 +313,4 @@ password PASSWORD
         msg = helpMessage % values
         return msg
 
+
