@@ -67,18 +67,27 @@ class Plugin(kforge.plugin.base.ServicePlugin):
                 service, path
             )
             self.log(msg)
+            self.buildAndReloadApacheConfig()
 
     def onServicePurge(self, service):
         super(Plugin, self),onServicePurge(service)
         # TODO: Delete the Joomla database instance.
     
-    def getApacheConfig(self, service):
-        result = ''' 
-            Alias %(urlPath)s %(fileSystemPath)s
-            <Location %(urlPath)s/administrator>
-                %(accessControl)s
-            </Location>'''
-        return result
+    def getApacheConfig(self, service, configVars):
+        apacheConfigTmpl = """
+<IfModule mod_wsgi.c>
+Alias %(urlPath)s %(fileSystemPath)s
+<Location %(urlPath)s/administrator>
+%(modWsgiAccessControl)s
+</Location>
+IfModule>
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
+%(modPythonAccessControl)s
+</IfModule>
+</IfModule>
+"""
+        return apacheConfigTmpl % configVars
 
     def backup(self, service, backupPathBuilder):
         # this is not great at present since we do a backup of the **whole**
