@@ -1,7 +1,6 @@
 import kforge.plugin.base
-import kforge.utils.mailer
 from kforge.dictionarywords import *
-from kforge.command.emailpassword import EmailCommand
+from dm.command.emailpassword import EmailCommand
 
 # Todo: Move to command module.
 class EmailNotifyCommand(EmailCommand):
@@ -28,9 +27,10 @@ class EmailNotifyCommand(EmailCommand):
     def getMessageToList(self):
         adminRole = self.registry.roles['Administrator']
         emailSet = set()
-        for person in self.registry.persons:
-            if person.role == adminRole and person.email:
-                emailSet.add(person.email)
+        for person in self.registry.people:
+            emailAddress = person.getEmailAddress()
+            if person.role == adminRole and emailAddress:
+                emailSet.add(emailAddress)
         if self.domainObject.meta.name == 'Project':
             project = self.domainObject
         elif self.domainObject.meta.name == 'Service':
@@ -41,8 +41,9 @@ class EmailNotifyCommand(EmailCommand):
             project = None
         if project != None:
             for member in project.members:
-                if member.role == adminRole and member.person.email:
-                    emailSet.add(member.person.email)
+                emailAddress = member.person.getEmailAddress()
+                if member.role == adminRole and emailAddress:
+                    emailSet.add(emailAddress)
         return emailSet
 
 
