@@ -18,20 +18,46 @@ You can enable, disable, and show status in the usual way.
   $ kforge-admin plugin show svn
   $ kforge-admin plugin disable svn
 """
+import kforge.plugin.base
+import kforge.utils.backup
+from kforge.dictionarywords import *
+from kforge.plugin.ssh import SSH_USER_NAME
+from kforge.plugin.ssh import SSH_HOST_NAME
+from kforge.plugin.base import dictionary, setWord
 import os
 import commands
 import shutil
 
-import kforge.plugin.base
-import kforge.utils.backup
-from kforge.dictionary import *
+SVN_ADMIN_SCRIPT = ExecutableFileSetting('svn.admin_script', umask=0o22)
+SVN_WSGI_SCRIPT_PATH = WsgiScriptSetting('svn.wsgi_script')
+SVN_WSGI_PROCESS_GROUP = StringSetting('svn.wsgi_process_group')
+SVN_CGI_SCRIPT_PATH = CgiScriptSetting('svn.cgi_script')
+SVN_VIEWVC_IS_ENABLED = StringSetting('svn.enable_viewvc')
+SVN_VIEWVC_LIB_PATH = DirSetting('svn.viewvc_lib')
+SVN_VIEWVC_MEDIA_PATH = DirSetting('svn.viewvc_media_dir')
+SVN_VIEWVC_TEMPLATE_PATH = DirSetting('svn.viewvc_template_dir')
+defaultWsgiScript = os.path.join(dictionary[FILESYSTEM_PATH], 'wsgi', 'svn.wsgi')
+defaultCgiScript = os.path.join(dictionary[FILESYSTEM_PATH], 'cgi', 'svn.cgi')
+setWord(SVN_ADMIN_SCRIPT, '/usr/bin/svnadmin')
+setWord(SVN_WSGI_SCRIPT_PATH, defaultWsgiScript)
+setWord(SVN_WSGI_PROCESS_GROUP, dictionary[WSGI_PROCESS_GROUP])
+setWord(SVN_CGI_SCRIPT_PATH, defaultCgiScript)
+setWord(SVN_VIEWVC_IS_ENABLED, '')
+setWord(SVN_VIEWVC_LIB_PATH, '/usr/lib/viewvc/lib')
+setWord(SVN_VIEWVC_MEDIA_PATH, '/usr/share/viewvc/docroot')
+setWord(SVN_VIEWVC_TEMPLATE_PATH, '/etc/viewvc/templates')
+
 
 class Plugin(kforge.plugin.base.ServicePlugin):
     "Subversion plugin."
+
+    settings = [SVN_ADMIN_SCRIPT, SVN_WSGI_SCRIPT_PATH,
+        SVN_WSGI_PROCESS_GROUP, SVN_CGI_SCRIPT_PATH, SVN_VIEWVC_IS_ENABLED,
+        SVN_VIEWVC_LIB_PATH, SVN_VIEWVC_MEDIA_PATH, SVN_VIEWVC_TEMPLATE_PATH]
     
     def __init__(self, domainObject):
         super(Plugin, self).__init__(domainObject)
-        self.utils = SvnUtils()
+        self.utils = SvnUtils(umask=self.dictionary.getUmask())
    
     def checkDependencies(self):
         errors = []
@@ -40,12 +66,13 @@ class Plugin(kforge.plugin.base.ServicePlugin):
         (status, output) = commands.getstatusoutput(cmd) 
         if status:
             errors.append("Couldn't find Subversion admin script '%s' on path." % svnadminPath)
-        if not os.path.isdir(self.dictionary[SVN_VIEWVC_LIB_PATH]):
-            errors.append("Couldn't find ViewVC library: %s" % self.dictionary[SVN_VIEWVC_LIB_PATH])
-        if not os.path.isdir(self.dictionary[SVN_VIEWVC_TEMPLATE_PATH]):
-            errors.append("Couldn't find ViewVC templates: %s" % self.dictionary[SVN_VIEWVC_TEMPLATE_PATH])
-        if not os.path.isdir(self.dictionary[SVN_VIEWVC_MEDIA_PATH]):
-            errors.append("Couldn't find ViewVC static media: %s" % self.dictionary[SVN_VIEWVC_MEDIA_PATH])
+        if self.dictionary[SVN_VIEWVC_IS_ENABLED]:
+            if not os.path.isdir(self.dictionary[SVN_VIEWVC_LIB_PATH]):
+                errors.append("Couldn't find ViewVC library: %s" % self.dictionary[SVN_VIEWVC_LIB_PATH])
+            if not os.path.isdir(self.dictionary[SVN_VIEWVC_TEMPLATE_PATH]):
+                errors.append("Couldn't find ViewVC templates: %s" % self.dictionary[SVN_VIEWVC_TEMPLATE_PATH])
+            if not os.path.isdir(self.dictionary[SVN_VIEWVC_MEDIA_PATH]):
+                errors.append("Couldn't find ViewVC static media: %s" % self.dictionary[SVN_VIEWVC_MEDIA_PATH])
         return errors
 
     def showDepends(self):
@@ -53,18 +80,24 @@ class Plugin(kforge.plugin.base.ServicePlugin):
         cmd = 'which %s' % svnadminPath
         (statusWhichSvnadmin, outputWhichSvnadmin) = commands.getstatusoutput(cmd) 
 
-        hasViewvcLibrary= os.path.isdir(self.dictionary[SVN_VIEWVC_LIB_PATH])
-        hasViewvcTemplate = os.path.isdir(self.dictionary[SVN_VIEWVC_TEMPLATE_PATH])
-        hasViewvcMedia = os.path.isdir(self.dictionary[SVN_VIEWVC_MEDIA_PATH])
-        return [
+        depends = [
             "Subversion admin: %s" % (statusWhichSvnadmin and "Not found!" or ("Found OK. %s" % outputWhichSvnadmin)),
-            "ViewVC library: %s" % (hasViewvcLibrary and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_LIB_PATH]) or "Not found!"),
-            "ViewVC template: %s" % (hasViewvcTemplate and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_TEMPLATE_PATH]) or "Not found!"),
-            "ViewVC media: %s" % (hasViewvcMedia and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_MEDIA_PATH]) or "Not found!"),
+        ]
+        if self.dictionary[SVN_VIEWVC_IS_ENABLED]:
+            hasViewvcLibrary= os.path.isdir(self.dictionary[SVN_VIEWVC_LIB_PATH])
+            hasViewvcTemplate = os.path.isdir(self.dictionary[SVN_VIEWVC_TEMPLATE_PATH])
+            hasViewvcMedia = os.path.isdir(self.dictionary[SVN_VIEWVC_MEDIA_PATH])
+            depends.extend([
+                "ViewVC library: %s" % (hasViewvcLibrary and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_LIB_PATH]) or "Not found!"),
+                "ViewVC template: %s" % (hasViewvcTemplate and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_TEMPLATE_PATH]) or "Not found!"),
+                "ViewVC media: %s" % (hasViewvcMedia and ("Found OK. %s" % self.dictionary[SVN_VIEWVC_MEDIA_PATH]) or "Not found!"),
+            ])
+        depends.extend([
             "Apache dav module: %s" % "Please check this module is enabled.",
             "Apache dav_fs module: %s" % "Please check this module is enabled.",
             "Apache dav_svn module: %s" % "Please check this module is enabled.",
-        ]
+        ])
+        return depends
     showDepends = classmethod(showDepends)
  
     def onServiceCreate(self, service):
@@ -76,85 +109,101 @@ class Plugin(kforge.plugin.base.ServicePlugin):
             self.logger.info(msg)
             self.paths.assertServiceFolder(service)
             self.utils.createRepository(servicePath)
+            self.buildAndReloadApacheConfig()
 
-    def getApacheConfig(self, service):
-        viewvcServicePathParent =  "%%(uriPrefix)s/%s/viewvc" % service.project.name
-        viewvcServicePath =  "%s/%s" % (viewvcServicePathParent, service.name)
-        viewvcMediaFolder = '/usr/share/viewvc/docroot'
-        viewvcMediaFolder = self.dictionary[SVN_VIEWVC_MEDIA_PATH]
-        # Only need this Alias directive once per project, but are generating
-        # it for each service. So use AliasMatch so that Apache does not print warnings.
-        # Same goes for the WSGIScript Alias, but Apache doesn't print warnings about that.
-        # Todo: Fixup ViewVC static file configuration (perhaps make it common config,
-        # and somehow setup the service environment to point to a common resource).
+    def getApacheConfig(self, service, configVars):
+        apacheConfigTmpl = '\n'
+        configVars['serviceName'] = service.name
+        configVars['projectName'] = service.project.name
+        if self.dictionary[SVN_VIEWVC_IS_ENABLED]:
+            configVars['viewvcWsgiScriptPath'] = self.dictionary[SVN_WSGI_SCRIPT_PATH]
+            configVars['viewvcCgiScriptPath'] = self.dictionary[SVN_GGI_SCRIPT_PATH]
+            viewvcServicePath =  "%(uriPrefix)s/%(projectName)s/viewvc/%(serviceName)s" % configVars
+            configVars['viewvcServicePath'] = viewvcServicePath
+            configVars['svnWsgiProcessGroup'] = self.dictionary[SVN_WSGI_PROCESS_GROUP]
+            configVars['viewvcMediaFolder'] = self.dictionary[SVN_VIEWVC_MEDIA_PATH]
+            # Want to do: <LocationMatch ^%(urlPath)s(!?viewvc)> but mod_dav_svn gets the paths wrong.
+            apacheConfigTmpl += """
+AliasMatch ^%(uriPrefix)s/%(projectName)s/viewvc\/\*docroot\*((?!%(serviceName)s).*) %(viewvcMediaFolder)s$1
 
-        viewvcRedirect = """
+<IfModule mod_wsgi.c>
+WSGIScriptAlias %(uriPrefix)s/%(projectName)s/viewvc %(viewvcWsgiScriptPath)s
+WSGIApplicationGroup %%{GLOBAL}
+WSGIProcessGroup %(svnWsgiProcessGroup)s
+</IfModule>
+
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
+ScriptAlias %(viewvcServicePath)s %(viewCgiScriptpath)s
+</IfModule>
+</IfModule>
 
-AliasMatch ^%(uriPrefix)s/"""+service.project.name+"/viewvc\/\*docroot\*((?!"+service.name+").*) "+viewvcMediaFolder+"""$1
 <IfModule mod_rewrite.c>
 RewriteEngine on
 RewriteCond %%{HTTP_USER_AGENT}  Lynx               [OR]
 RewriteCond %%{HTTP_USER_AGENT}  Mozilla            [OR]
 RewriteCond %%{HTTP_USER_AGENT}  Links              [OR]
 RewriteCond %%{HTTP_USER_AGENT}  w3m
-RewriteRule ^%(urlPath)s$       """+viewvcServicePath+"""/  [R,L]
-</IfModule>"""
-        # Want to do: <LocationMatch ^%(urlPath)s(!?viewvc)> but mod_dav_svn gets the paths wrong.
-        if self.dictionary[SVN_DAV_MOD_PYTHON_ACCESS_CONTROL]:
-            accessControl = '%(modPythonAccessControl)s'
-        else:
-            accessControl = '%(accessControl)s'
-        davsvnLocation = """
-<Location %(urlPath)s>
+RewriteRule ^%(urlPath)s$       %(viewvcServicePath)s/  [R,L]
+</IfModule>
+
+<IfModule mod_wsgi.c>
+<Location %(viewvcServicePath)s>
+SetEnv KFORGE_SVN_REPO_PATH %(fileSystemPath)s
+SetEnv KFORGE_SVN_SERVICE_NAME %(serviceName)s
+WSGIPassAuthorization On
+</Location>
+</IfModule>
+
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
+<Location %(viewvcServicePath)s>
+SetEnv KFORGE_SVN_REPO_PATH %(fileSystemPath)s
+SetEnv KFORGE_SVN_SERVICE_NAME %(serviceName)s
+%(modPythonAccessControl)s
+</Location>
+</IfModule>
+</IfModule>
+"""
+            apacheConfigTmpl += viewvcConfigTmpl
+        # Add configuration for DAV Svn.
+        apacheConfigTmpl += """
 <IfModule mod_dav.c>
+<IfModule mod_dav_svn.c>
+<IfModule mod_wsgi.c>
+<Location %(urlPath)s>
 DAV svn 
 SVNPath %(fileSystemPath)s
-"""+accessControl+"""
+# Prefer mod_python access control (avoids unnecessary prompt).
+<IfModule mod_python.c>
+# Don't cache the model in mod_python since Web UI is running in mod_wsgi.
+PYTHONOPTION AVOID_ENABLING_MODEL_CACHE True
+%(modPythonAccessControl)s
 </IfModule>
-</Location>"""
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            wsgiScriptPath = self.dictionary[SVN_WSGI_SCRIPT_PATH]
-            viewvcScriptAlias = """
-WSGIScriptAlias """+viewvcServicePathParent+" "+wsgiScriptPath+"""
-WSGIApplicationGroup %%{GLOBAL}
-WSGIProcessGroup """ + self.dictionary[SVN_WSGI_PROCESS_GROUP]
-            viewvcLocation = """
-<Location """+viewvcServicePath+""">
-SetEnv KFORGE_SVN_REPO_PATH %(fileSystemPath)s
-SetEnv KFORGE_SVN_SERVICE_NAME """+service.name+"""
-WSGIPassAuthorization On
-</Location>"""
-        elif self.dictionary[APACHE_PYTHON_MODULE] == 'mod_python':
-            cgiScriptPath = self.dictionary[SVN_CGI_SCRIPT_PATH]
-            viewvcScriptAlias = """
-ScriptAlias """+viewvcServicePath+" "+cgiScriptPath
-            viewvcLocation = """
-<Location """+viewvcServicePath+""">
-SetEnv KFORGE_SVN_REPO_PATH %(fileSystemPath)s
-SetEnv KFORGE_SVN_SERVICE_NAME """+service.name+"""
-%(accessControl)s
-</Location>"""
-        config = viewvcScriptAlias + viewvcRedirect + viewvcLocation + davsvnLocation
-        return config
-
-    helpMessage = '''
-<p>This service provides a <a href="http://subversion.tigris.org/">Subversion</a> repository located at:</p>
-<p style="text-align: center"><a href="%(url)s">%(url)s</a></p>
-<p>A Subversion repository is a sharable versioned filesystem. To find out more about them see the <a href="http://svnbook.red-bean.com/">Subversion book</a>. You can access the material in the repository in one of the following ways (provided you have read permissions):</p>
-<ul>
-    <li>Browse the material online by following the link to the Subversion repository in your browser.</li>
-    <li>'Check out' the repository with a suitable Subversion client. For example, if you are using the command line client, you would do:<br />
-    <code>$ svn checkout %(url)s --username username</code><br /></li>
-</ul>
-<p>For more information on the command line client see the Subversion book (chapter 3). Alternatively there is <a href="http://tortoisesvn.tigris.org/">Tortoise SVN</a>, a GUI client that integrates with Windows explorer. It has a <a href="http://tortoisesvn.net/doc_release">detailed manual</a> with chapter 5 covering 'daily use' -- checking out, committing etc.</p>
-<p>Note that if you wish to 'commit' work to the repository you must have a role on the project that includes 'write' permissions on Subversion services.</p>
-'''
+<IfModule !mod_python.c>
+%(modWsgiAccessControl)s
+</IfModule>
+</Location>
+</IfModule>
+<IfModule !mod_wsgi.c>
+<IfModule mod_python.c>
+<Location %(urlPath)s>
+DAV svn 
+SVNPath %(fileSystemPath)s
+%(modPythonAccessControl)s
+</Location>
+</IfModule>
+</IfModule>
+</IfModule>
+</IfModule>
+"""
+        return apacheConfigTmpl % configVars
 
     def buildWsgiFile(self):
-        if self.dictionary[APACHE_PYTHON_MODULE] == 'mod_wsgi':
-            wsgiScriptPath = self.dictionary[SVN_WSGI_SCRIPT_PATH]
-            wsgiScriptContent = self.createWsgiScriptContent()
-            self.writeFile(wsgiScriptPath, wsgiScriptContent, 'Subversion WSGI script')
+        path = self.dictionary[SVN_WSGI_SCRIPT_PATH]
+        content = self.createWsgiScriptContent()
+        purpose = 'Subversion WSGI script'
+        self.filesystem.writeFileIfChanged(path, content, purpose)
 
     def createWsgiScriptBody(self, pythonPathActivation):
         viewvcLibPath = self.dictionary[SVN_VIEWVC_LIB_PATH]
@@ -190,6 +239,7 @@ def dispatch_viewvc_request(environ, start_response):
     serviceName = environ.pop('KFORGE_SVN_SERVICE_NAME')
     cfg.general.svn_roots = {serviceName: repositoryPath}
     cfg.options.svn_roots = {serviceName: repositoryPath}
+    cfg.options.allowed_views.append('co')
 
     #cfg.conf_path = ' ' # Stops the viewvc's "hack_..." method failing. :-)
     cfg.options.hide_cvs_root = 1
@@ -203,17 +253,19 @@ def application(environ, start_response):
         return wsgiScriptBody
 
     def buildCgiFile(self):
-        cgiScriptPath = self.dictionary[SVN_CGI_SCRIPT_PATH]
-        cgiScriptContent = self.createCgiScriptContent()
-        self.writeFile(cgiScriptPath, cgiScriptContent, 'Subversion CGI script')
-        # Make CGI script executable.
-        os.chmod(cgiScriptPath, 0755)
+        path = self.dictionary[SVN_CGI_SCRIPT_PATH]
+        content = self.createCgiScriptContent()
+        purpose = 'Subversion CGI script'
+        self.filesystem.writeCgiFile(path, content, purpose)
 
     def createCgiScriptContent(self):
         # Todo: Fixup so it uses virtualenv or PYTHONPATH, if they exist.
         viewvcLibPath = self.dictionary[SVN_VIEWVC_LIB_PATH]
         viewvcTemplatePath = self.dictionary[SVN_VIEWVC_TEMPLATE_PATH]
-        cgiScriptContent = '''#!/usr/bin/env python
+        pythonBinPath = "/usr/bin/env python"
+        if self.dictionary[VIRTUALENVBIN_PATH]:
+            pythonBinPath = os.path.join(self.dictionary[VIRTUALENVBIN_PATH], 'python')
+        cgiScriptContent = '''#!'''+pythonBinPath+'''
 
 LIBRARY_DIR = r"'''+viewvcLibPath+'''"
 
@@ -246,18 +298,12 @@ viewvc.main(server, cfg)
 '''
         return cgiScriptContent
 
-    def getUserHelp(self, service, serviceLocation):
-        values = { 'url' : serviceLocation }
-        msg = self.helpMessage % values
-        return msg
-    
     def getUserHelp(self, service, serviceLocation):
         values = {'httpUrl' : serviceLocation}
         if 'ssh' in self.registry.plugins:
-            import getpass
             sshUrl = 'svn+ssh://%(sshUser)s@%(sshHost)s/%(project)s/%(service)s'
             sshUrl %= {
-                'sshUser': self.dictionary[SSH_USER_NAME] or getpass.getuser(),
+                'sshUser': self.dictionary[SSH_USER_NAME],
                 'sshHost': self.dictionary[SSH_HOST_NAME],
                 'project': service.project.name,
                 'service': service.name
@@ -317,7 +363,7 @@ viewvc.main(server, cfg)
 
 class SvnUtils(object):
     
-    def __init__(self, parentPath=''):
+    def __init__(self, parentPath='', umask=0o077):
         """
         @parentPath: string representing parent path to use when creating
                      repositories. If not defined then defaults to empty string
@@ -326,6 +372,7 @@ class SvnUtils(object):
         # NB: Only used in svntest.py.
         # Todo: Either move test path switching to dictionary manipulations, or create subclass in test unit and test that.
         self.parentPath = parentPath  
+        self.umask = umask
    
     def createRepository(self, path):
         """
@@ -340,10 +387,10 @@ class SvnUtils(object):
         status, output = commands.getstatusoutput(cmd)
         if status:
             raise Exception('Subversion command error: %s: %s' % (cmd, output))
-        # Make the rep-cache.db file group writable since svnadmin doesn't ensure this.
+        # Make the rep-cache.db file group writable since svnadmin doesn't.
         repCachePath = '%s/db/rep-cache.db' % (path)
         if os.path.exists(repCachePath):
-            os.chmod(repCachePath, 436)
+            os.chmod(repCachePath, 0660 & ~ self.umask)
 
     def deleteRepository(self, path):
         """
