@@ -4,7 +4,6 @@ import shutil
 import tempfile
 
 import kforge.plugin.wordpress
-import kforge.filesystem
 from kforge.ioc import *
 from kforge.testunit import TestCase
 
@@ -15,8 +14,6 @@ def suite():
     ]
     return unittest.TestSuite(suites)
 
-fsPaths = kforge.filesystem.FileSystemPathBuilder()
-
 class PluginTest(TestCase):
     
     def setUp(self):
@@ -26,13 +23,12 @@ class PluginTest(TestCase):
         self.plugin = self.registry.plugins['wordpress']
         self.pluginSystem = self.plugin.getSystem()
         self.project = self.registry.projects['annakarenina']
-        services = self.plugin.services[self.project]
         try:
-            self.service = services.create('wordpress')
+            self.service = self.plugin.services.create(name='wordpress', project=self.project)
         except:
-            service = services['wordpress']
-            service.delete()
-            service.purge()
+            for service in self.plugin.services.findDomainObjects(name='wordpress', project=self.project):
+                service.delete()
+                service.purge()
             raise
     
     def tearDown(self):
@@ -43,11 +39,6 @@ class PluginTest(TestCase):
     def testServicesPaths(self):
         self.failUnless(self.service.hasDir(), self.service.getDirPath())
     
-    def testGetApacheConfig(self):
-        config = self.plugin.getSystem().getApacheConfig(self.service)
-        exp = 'Alias %(urlPath)s '
-        self.failUnless(exp in config)
-    
     def test_backup(self):
         tmpdir = tempfile.mkdtemp(prefix='kforge-wordpress-')
         class backupPathBuilder:
