@@ -31,12 +31,12 @@ class Plugin(kforge.plugin.base.ServicePlugin):
         if self.isOurs(service):
             fsPath = self.paths.getServicePath(service)
             self.paths.assertFolder(fsPath, 'project services')
+            self.buildAndReloadApacheConfig()
     
-    def getApacheConfig(self, service):
-        """
-        Note: allow access to everyone -- even guest
-        """
-        fsPath = self.paths.getServicePath(service)
-        result = 'Alias %(urlPath)s ' + fsPath + '\n'
-        return result
-
+    def getApacheConfig(self, service, configVars):
+        # It's a Web site, so allow access to everyone -- even guest
+        configVars['fsPath'] = self.paths.getServicePath(service)
+        apacheConfigTmpl = """
+Alias %(urlPath)s %(fsPath)s
+"""
+        return apacheConfigTmpl % configVars
