@@ -1,7 +1,7 @@
 import unittest
-
 from kforge.testunit import TestCase
-import kforge.filesystem
+from kforge.filesystem import FileSystem
+import os
 
 def suite():
     suites = [
@@ -23,7 +23,7 @@ class PluginTest(TestCase):
             service.purge()
         self.project.services.create('www', plugin=self.plugin)
         self.service = self.project.services['www']
-        self.fsPathBuilder = kforge.filesystem.FileSystemPathBuilder()
+        self.filesystem = FileSystem()
     
     def tearDown(self):
         self.service.delete()
@@ -31,10 +31,7 @@ class PluginTest(TestCase):
     
     def testService(self):
         # Check the filesystem has been setup.
-        fsPath = self.fsPathBuilder.getServicePath(self.service)
-        self.failUnless(fsPath)
-        # Check the Apache configuration looks okay.
-        apacheConfig = self.plugin.getSystem().getApacheConfig(self.service)
-        expect = 'Alias %(urlPath)s ' + fsPath + '\n'
-        self.assertEquals(apacheConfig, expect)
+        path = self.filesystem.getServicePath(self.service)
+        self.failUnless(path)
+        self.failUnless(os.path.exists(path))
 
