@@ -7,7 +7,6 @@ import os
 
 from kforge.testunit import *
 import kforge.plugin.basetest
-import kforge.plugin.apacheconfigtest
 import kforge.plugin.accesscontroltest
 import kforge.plugin.wwwtest
 import kforge.plugin.davtest
@@ -16,12 +15,12 @@ import kforge.plugin.tractest
 import kforge.plugin.mointest
 import kforge.plugin.wordpresstest
 import kforge.plugin.mailmantest
+import kforge.plugin.mercurialtest
 import kforge.plugin.notifytest
 
 def suite():
     suites = [
         kforge.plugin.basetest.suite(),
-        kforge.plugin.apacheconfigtest.suite(),
         kforge.plugin.accesscontroltest.suite(),
         kforge.plugin.wwwtest.suite(),
         kforge.plugin.davtest.suite(),
@@ -30,6 +29,7 @@ def suite():
         kforge.plugin.mointest.suite(),
         kforge.plugin.wordpresstest.suite(),
         kforge.plugin.mailmantest.suite(),
+        kforge.plugin.mercurialtest.suite(),
         kforge.plugin.notifytest.suite(),
     ]
     return unittest.TestSuite(suites)
