@@ -17,6 +17,6 @@ class TestServiceCRUD(KuiTestCase):
         self.registerProject()
 
     def test_project_services(self):
-        self.getAssertContent(self.urlProjectServices, 'Project Services')
-        self.getAssertContent(self.urlProjectServiceCreate, 'Create new service')
+        self.getAssertContent(self.urlProjectServices, 'Services')
+        self.getAssertContent(self.urlProjectServiceCreate, 'Add new service')
 
