@@ -38,10 +38,18 @@ class TestMercurial(PluginTestCase):
 
     def testMercurialClient(self):
         self.createService('mercurial', 'mercurial')
-        commitMsg = 'Just testing...'
+        commitMsg = 'Testing HTTP access client...'
         # Todo: Check the .hgrc file has necessary lines.
         self.mercurialCloneAddCommitPush('mercurial', commitMsg)
         self.setBasicAuthPerson()
         url = self.getServicePath('mercurial')
         self.getAssertContent(url, commitMsg)
 
+    def testPerformance(self):
+        self.createService('mercurial', 'mercurial2')
+        # Check performance for project developer.
+        self.changePersonProjectRole('Developer')
+        self.mercurialCloneAddCommitPush('mercurial2', 'Testing Mercurial HTTP access (push performance)...', checkPushPerformance=30)
+        self.mercurialCloneAddCommitPush('mercurial2', 'Testing Mercurial HTTP access (clone performance)...', checkClonePerformance=30)
+        
+
