@@ -7,7 +7,7 @@ import kforge.ioc
 import kforge.exceptions
 
 import routes
-from kforge.dictionarywords import DOMAIN_NAME
+from kforge.dictionarywords import SITE_HOST
 from kforge.dictionarywords import HTTP_PORT
 from kforge.dictionarywords import URI_PREFIX
 
@@ -24,16 +24,16 @@ class UrlScheme(object):
 
         * media
         * system_admin
-        * person
-        * project
-        * project_admin
-        * project_service
+        * people
+        * projects
+        * projects_admin
+        * projects_service
 
     """
 
     def __init__(self):
         self.dictionary = kforge.ioc.RequiredFeature('SystemDictionary')
-        self.fqdn = self.dictionary[DOMAIN_NAME]
+        self.fqdn = self.dictionary[SITE_HOST]
         if self.fqdn.endswith('/'):
             self.fqdn= self.host[:-1]
         self.port = self.dictionary[HTTP_PORT]
@@ -61,6 +61,8 @@ class UrlScheme(object):
 
     def _configure_mapper(self):
         self.mapper.connect('home', '')
+        self.mapper.connect('api', 'api', controller='api')
+        self.mapper.connect('about', 'about', controller='about')
         self.mapper.connect('feed', 'feed', controller='feed')
         self.mapper.connect('media', 'media*offset', controller='media',
                 offset='')
@@ -68,52 +70,54 @@ class UrlScheme(object):
                 action='logout')
         self.mapper.connect('login', 'login', controller='account',
                 action='login')
+        self.mapper.connect('recover', 'recover', controller='account',
+                action='recover')
         self.mapper.connect('access_denied', 'accessDenied',
                 controller='accessDenied')
-        self.mapper.connect('person', 'person/:action/:id',
-                controller='person', action='index',
+        self.mapper.connect('people', 'people/:action/:id',
+                controller='people', action='index',
                 requirements={'action': 'index|create|search|find|list|home'}
                 )
-        self.mapper.connect('person', 'person/:id/:action',
-                controller='person', action='read',
-                requirements={'action': 'edit|delete|read'}
+        self.mapper.connect('people', 'people/:id/:action',
+                controller='people', action='read',
+                requirements={'action': 'edit|delete|read|tickets'}
                 )
-        self.mapper.connect('person.admin',
-                'person/:person/:subcontroller/:action',
-                controller='person.admin', id=None, 
+        self.mapper.connect('people.admin',
+                'people/:person/:subcontroller/:action',
+                controller='people.admin', id=None, 
                 requirements={'subcontroller': 'sshKeys',
                     'action': 'create'}
                 )
-        self.mapper.connect('person.admin',
-                'person/:person/:subcontroller/:id/:action',
-                controller='person.admin', action='read',
+        self.mapper.connect('people.admin',
+                'people/:person/:subcontroller/:id/:action',
+                controller='people.admin', action='read',
                 requirements={'subcontroller': 'sshKeys',
                     'action': 'edit|delete|read'}
                 )
-        self.mapper.connect('project', 'project/:action/:id',
-                controller='project', action='index',
+        self.mapper.connect('projects', 'projects/:action/:id',
+                controller='projects', action='index',
                 requirements={'action': 'index|create|search|find|list|home'}
                 )
-        self.mapper.connect('project', 'project/:id/:action',
-                controller='project', action='read',
+        self.mapper.connect('projects', 'projects/:id/:action',
+                controller='projects', action='read',
                 requirements={'action': 'edit|delete|read|join'}
                 )
-        self.mapper.connect('project.admin',
-                'project/:project/:subcontroller/:action',
-                controller='project.admin', id=None, 
+        self.mapper.connect('projects.admin',
+                'projects/:project/:subcontroller/:action',
+                controller='projects.admin', id=None, 
                 requirements={'subcontroller': 'services|members',
                     'action': 'create'}
                 )
-        self.mapper.connect('project.admin',
-                'project/:project/:subcontroller/:id/:action',
-                controller='project.admin', action='read',
+        self.mapper.connect('projects.admin',
+                'projects/:project/:subcontroller/:id/:action',
+                controller='projects.admin', action='read',
                 requirements={'subcontroller': 'services|members',
                     'action': 'edit|delete|read|reject|approve'}
                 )
         # do not really need the controller here since this map is only used in
         # url generation but need it for routes
-        self.mapper.connect('project.service', ':project/:service',
-                controller='project.service', service=None)
+        self.mapper.connect('projects.service', ':project/:service',
+                controller='projects.service', service=None)
         # this is not fully complete as we don't properly deal with subitems.
         # would like to have several maps and recurse but routes does not
         # support multiple mappers very nicely (if only url_for had a a config
@@ -138,13 +142,13 @@ class UrlScheme(object):
         return url
 
     def getServicePath(self, service):
-        return self.url_for('project.service',
+        return self.url_for('projects.service',
             project=service.project.name,
             service=service.name
         )
 
     def getServiceUrl(self, service):
-        return self.url_for_qualified('project.service',
+        return self.url_for_qualified('projects.service',
             project=service.project.name,
             service=service.name
         )
