@@ -38,7 +38,7 @@ class DatabaseTest(unittest.TestCase):
         # do a crude check that this has run
         import kforge
         app = kforge.getA()
-        admin = app.registry.persons['admin']
+        admin = app.registry.people['admin']
 
 #    def test_rebuild(self):
 #        self.db.rebuild()
